@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -127,12 +127,10 @@ static CSTR table_string[] = {
 	{ CFG_DIR2,			"HOME",	"",				"", "" },
 	{ CFG_FMT_DATE,		"AUTO",	"",				"",	"" },
 	{ CFG_HELPFILE,		"NONE",	"",				"", "" },
-	{ CFG_LAYOUT1,		0,
-	  "$d $s $>$t $M $*|  $p $o $L", "", "" },
-	{ CFG_LAYOUT2,		0,
-	  "$d $s $t $*|  $p $o", "", "" },
+	{ CFG_LAYOUT1,		0, "$d $S $>$t $M $*|  $p $o $L", "", "" },
+	{ CFG_LAYOUT2,		0, "$d $S $t $*|  $p $o", "", "" },
 	{ CFG_LAYOUT3,		0,
-	  "$d $s $p $>$t $*|  $m $o atime=$a ctime=$i links=$l", "", "" },
+	  "$p $o $s $d $>$t $*|  mode=$m atime=$a ctime=$i links=$l", "", "" },
 	{ CFG_PROMPT,		"AUTO",	"",				"", "" },
 	{ CFG_QUOTE,		0,		"",				"", "" },
 	{ CFG_SHELLPROG,	"AUTO",	"",				"", "" }
@@ -161,8 +159,8 @@ static struct {
 		"How many lines are occupied by the input line" },
 	{ CFG_COLLATION,	"Appearance: Filename collation sequence" },
 	{ CFG_D_SIZE,		"Directory panel size (AUTO = screen size)" },
-	{ CFG_DIR2,			"Secondary file panel's directory "
-						"(HOME = home directory)" },
+	{ CFG_DIR2,			"Secondary file panel's initial directory "
+						"(HOME = home dir)" },
 	{ CFG_FRAME,		"Appearance: "
 		"Panel frame: ----- or ===== or line graphics" },
 	{ CFG_FMT_DATE,		"Appearance: "
@@ -481,8 +479,6 @@ config_initialize(void)
 	}
 
 	if (clex_data.admin) {
-		panel_cfg.pd->min = -2;
-		panel_cfg.pd->type = PANEL_TYPE_CFG_ADMIN;
 		/*
 		 * check user's access rights
 		 * this may create an empty file, never mind
@@ -503,8 +499,6 @@ config_initialize(void)
 		read_configuration(CONFIG_FILE,0);
 	}
 	else {
-		panel_cfg.pd->min = -3;
-		panel_cfg.pd->type = PANEL_TYPE_CFG;
 		/* user's configuration file name */
 		pathname_set_directory(clex_data.homedir);
 		user_config_file = estrdup(pathname_join(".clexrc"));
@@ -777,6 +771,11 @@ config_apply(void)
 		sort_files();
 		filepos_set();
 		/* sort the secondary panel */
+		/*
+		 * warning: during the sorting of the secondary panel,
+		 * the primary panel does not correspond with the current
+		 * working directory
+		 */
 		ppanel_file = ppanel_file->other;
 		filepos_save();
 		sort_files();
@@ -794,34 +793,28 @@ config_apply(void)
 		curses_restart();
 }
 
+void
+cx_config_admin_save(void)
+{
+	config_changes();
+	config_save();
+}
+
+void
+cx_config_save(void)
+{
+	/* cursor -2 = ACCEPT, -1 = SAVE */
+	config_changes();
+	config_apply();
+	if (panel_cfg.pd->curs == -1)
+		config_save();
+}
+
 void
 cx_config_enter(void)
 {
 	CNUM *pnum;
 
-	if (panel_cfg.pd->curs < 0) {
-		if (clex_data.admin) {
-			/* admin mode:  -2 = EXIT, -1 = SAVE */
-			if (panel_cfg.pd->curs == -2)
-				next_mode = MODE_SPECIAL_QUIT;
-			else {
-				config_changes();
-				config_save();
-			}
-		}
-		else {
-			/* normal config: -3 = CANCEL, -2 = ACCEPT, -1 = SAVE */
-			if (panel_cfg.pd->curs >= -2) {
-				config_changes();
-				config_apply();
-				if (panel_cfg.pd->curs == -1)
-					config_save();
-			}
-			next_mode = MODE_SPECIAL_RETURN;
-		}
-		return;
-	}
-
 	if (config[panel_cfg.pd->curs].isnum) {
 		pnum = config[panel_cfg.pd->curs].table;
 		if (pnum->desc[0]) {
