@@ -8,6 +8,8 @@ extern void cx_config_num_enter(void);
 extern void cx_config_str_enter(void);
 extern void cx_config_default(void);
 extern void cx_config_original(void);
+extern void cx_config_save(void);
+extern void cx_config_admin_save(void);
 
 #define config_num(X) (*(const int *)pcfg[X])
 #define config_str(X) ((const char *)pcfg[X])
