@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -27,6 +27,8 @@
 #define CH_ESC			'\033'						/* ASCII escape */
 #define CH_CTRL(X)		((X) & 0x1F)				/* ASCII ctrl-X */
 #define IS_CHAR(X)		(((X) & 0xFF) == (X))		/* X is a char */
+/* CMP: no overflow and result is an int even if V1 and V2 are not */
+#define CMP(V1,V2) ((V1) == (V2) ? 0 : (V1) < (V2) ? -1 : 1)
 /* you can't do this in one pass: */
 #define STR(X)	STRINGIZE(X)
 #define STRINGIZE(X)	#X
@@ -47,7 +49,7 @@ typedef short int CODE;				/* usually some #define-d value */
  * value 0 is reserved, it means mode unchanged in control_loop()
  *   and also mode not set during startup
  */
-#define MODE_BM					 1
+#define MODE_BM_LIST			 1
 #define MODE_BM_MANAGER			 2
 #define MODE_BM_EDIT			 3
 #define MODE_CFG				 4
@@ -59,13 +61,14 @@ typedef short int CODE;				/* usually some #define-d value */
 #define MODE_DIR				10
 #define MODE_DIR_SPLIT			11
 #define MODE_FILE				12
-#define MODE_HELP				13
-#define MODE_HIST				14
-#define MODE_P_LOCATE			15	/* P = pattern */
-#define MODE_S_LOCATE			16	/* S = substring */
-#define MODE_MAINMENU			17
+#define MODE_GROUP				13
+#define MODE_HELP				14
+#define MODE_HIST				15
+#define MODE_MAINMENU			16
+#define MODE_PASTE				17
 #define MODE_SELECT				18
 #define MODE_SORT				19
+#define MODE_USER				20
 /* pseudo-modes */
 #define MODE_SPECIAL_QUIT		98
 #define MODE_SPECIAL_RETURN		99
@@ -95,17 +98,18 @@ typedef struct {
 	int panlines;		/* number of lines in the panel area */
 	int textline_area;	/* number of chars available for
 						   the textline (including the prompt) */
-	int date_len;		/* lenght of date/time field */
+	int date_len;		/* length of date/time field */
 } DISPLAY;
 
 typedef struct {
 	const char *login_at_host;	/* my name: user@host */
-	const char *shell;			/* my login shell */
+	char *shell;				/* my login shell */
 	const char *homedir;		/* my home directory */
 	pid_t pid;					/* process ID */
 	mode_t umask;				/* umask value */
 	FLAG isroot;				/* effective uid is 0(root) */
 	FLAG admin;					/* admin mode: clex -a */
+	CODE shelltype;				/* Bourne shell (0), C-shell (1), other (2) */
 } CLEX_DATA;
 
 /* description of an editing operation */
@@ -144,6 +148,16 @@ typedef struct {
 	EDIT_OP last_op;		/* last editing operation */
 } TEXTLINE;
 
+/* minimalistic version of TEXTLINE */
+/* currently used only for panel filters */
+#define INPUT_STR 25
+typedef struct {
+	char line[INPUT_STR];	/* user's input */
+	int size;				/* number of chars in the line */
+	int curs;				/* cursor position from 0 to 'size' */
+	FLAG changed;			/* 'line' has been modified */
+} INPUTLINE;
+
 /********************************************************************/
 
 /*
@@ -152,35 +166,55 @@ typedef struct {
  * the draw_panel_line() in inout.c
  */
 #define PANEL_TYPE_BM			 0
-#define PANEL_TYPE_BM_MANAGER	 1
-#define PANEL_TYPE_CFG			 2
-#define PANEL_TYPE_CFG_ADMIN	 3
-#define PANEL_TYPE_COMPARE		 4
-#define PANEL_TYPE_COMPL		 5
-#define PANEL_TYPE_DIR			 6
-#define PANEL_TYPE_DIR_SPLIT	 7
-#define PANEL_TYPE_FILE			 8
-#define PANEL_TYPE_HELP			 9
-#define PANEL_TYPE_HIST			10
-#define PANEL_TYPE_LOCATE		11
-#define PANEL_TYPE_MAINMENU		12
-#define PANEL_TYPE_SORT			13
+#define PANEL_TYPE_CFG			 1
+#define PANEL_TYPE_COMPARE		 2
+#define PANEL_TYPE_COMPL		 3
+#define PANEL_TYPE_DIR			 4
+#define PANEL_TYPE_DIR_SPLIT	 5
+#define PANEL_TYPE_FILE			 6
+#define PANEL_TYPE_GROUP		 7
+#define PANEL_TYPE_HELP			 8
+#define PANEL_TYPE_HIST			 9
+#define PANEL_TYPE_MAINMENU		10
+#define PANEL_TYPE_PASTE		11
+#define PANEL_TYPE_SORT			12
+#define PANEL_TYPE_USER			13
 #define PANEL_TYPE_NONE			99	/* not set (only during startup) */
 
+/*
+ * extra lines appear in a panel before the real first line,
+ * extra lines:  -MIN .. -1
+ * real lines:      0 .. MAX
+ */
+typedef struct {
+	const char *text;		/* text to be displayed in the panel */
+							/* default (if null): "Leave this panel" */
+	const char *info;		/* text to be displayed in the info line */
+	/* when this extra line is selected: */
+	CODE mode_next;			/* set next_mode to this mode and then ... */
+	void (*fn)(void); 		/* ... invoke this function */
+} EXTRA_LINE;
+
 /* description of a panel */
 typedef struct {
-	int cnt, top;		/* panel lines: total count, top of the screen */
-	int curs, min;		/* panel lines: cursor bar, top of the panel */
+	int cnt, top;	/* panel lines: total count, top of the screen */
+	int curs, min;	/* panel lines: cursor bar, top of the panel */
 	/*
-	 * panel line 'min' is used to insert extra lines
-     * before the actual first line which is always line number 0;
-	 * to insert N extra lines set 'min' to -N;
-	 * the number of extra lines is not included in 'cnt' 
+	 * 'min' is used to insert extra lines before the real first line
+     * which is always line number 0; to insert N extra lines set
+	 * 'min' to -N; the number of extra lines is not included in 'cnt' 
 	 */
-	CODE type;			/* panel type: one of PANEL_TYPE_XXX */
-	FLAG norev;			/* do not show the current line in reverse video */
+	CODE type;		/* panel type: one of PANEL_TYPE_XXX */
+	FLAG norev;		/* do not show the current line in reversed video */
+	EXTRA_LINE *extra;	/* extra panel lines */
+	INPUTLINE *filter;	/* filter (if applicable to this panel type) */
+	CODE filtering;		/* filter: 0 = off */
+						/* 1 = on - focus on the filter string */
+						/* 2 = on - focus on the command line */
 } PANEL_DESC;
 
+#define VALID_CURSOR(P) ((P)->cnt > 0 && (P)->curs >= 0 && (P)->curs < (P)->cnt)
+
 /********************************************************************/
 
 /*
@@ -215,9 +249,10 @@ typedef struct {
 /* text buffer sizes */			/* examples: */
 #define FE_LINKS_STR	 4		/* 1  999  max  */
 #define FE_TIME_STR		12		/* 12:34am  1.01.04  01-jan-2004 */
-#define FE_SIZE_DEV_STR	12		/* 122.351k  3.222.891K */
+#define FE_SIZE_DEV_STR	12		/* 3.222.891Ki */
 #define FE_MODE_STR 	 5		/* 0644 */
-#define FE_OWNER_STR	18		/* root:mail */
+#define FE_NAME_STR		10		/* root */
+#define FE_OWNER_STR	(2 * FE_NAME_STR)	/* root:mail */
 
 /*
  * file description - exhausting, isn't it ?
@@ -231,11 +266,12 @@ typedef struct {
 	off_t size;				/* file size */
 	dev_t devnum;			/* major/minor numbers (devices only) */
 	CODE file_type;			/* one of FT_XXX */
-	uid_t uid, gid;			/* owner & group */
+	uid_t uid, gid;			/* owner and group */
 	short int mode12;		/* file mode - low 12 bits */
 	unsigned int select:1;		/* flag: this entry is selected */
 	unsigned int symlink:1;		/* flag: it is a symbolic link */
 	unsigned int dotdir:2;		/* . (1) or .. (2) directory */
+	unsigned int fmatch:1;		/* flag: matches the filter */
 	/*
 	 * note: the structure members below are used
 	 * only when the file panel layout requires them
@@ -257,10 +293,18 @@ typedef struct ppanel_file {
 	struct ppanel_file *other;	/* primary <--> secondary panel ptr */
 	int selected;			/* number of selected entries */
 	FLAG expired;			/* expiration: panel needs re-read */
+	FLAG filtype;			/* filter type: 0 = substring, 1 = pattern */
+	int filt_cnt;			/* saved number of entries while filtering is on */
+	int filt_sel;			/* selected entries NOT matched by the filter */
 	int fe_alloc;			/* allocated FILE_ENTRies in 'files' below */
 	FILE_ENTRY **files;		/* main part: list of files in panel's
 							   working directory 'dir' */
 } PANEL_FILE;
+/*
+ * filter off: 0 .. cnt-1     = all file entries
+ * filter on:  0 .. cnt-1     = entries matching the filter expression
+ *           cnt .. filtcnt-1 = saved entries not matching the filter
+ */
 
 /********************************************************************/
 
@@ -271,14 +315,6 @@ typedef struct {
 
 /********************************************************************/
 
-typedef struct {
-	PANEL_DESC *pd;
-	int fe_alloc;			/* same meaning as in PANEL_FILE above */
-	FILE_ENTRY **files;
-} PANEL_LOCATE;
-
-/********************************************************************/
-
 /*
  * file sort order - if you change this, you must also update
  * panel initization in start.c and descriptions in inout.c
@@ -299,11 +335,11 @@ typedef struct {
 /********************************************************************/
 
 typedef struct {
-	const char *name;		/* directory name (see also shlen) */
-	int shlen;				/* PANEL_DIR: length of the repeating name part
-							     used for fancy formatting
-							   PANEL_DIR_SPLIT: length of the string 'name'
-								 which is NOT null terminated */
+	const char *name;	/* directory name (see also shlen) */
+	int shlen;			/* PANEL_DIR: length of the repeating name part
+							 (used for fancy formatting)
+						   PANEL_DIR_SPLIT: length of the string 'name'
+							 which is NOT null terminated */
 } DIR_ENTRY;
 
 typedef struct {
@@ -357,7 +393,7 @@ typedef struct {
 #define CFG_D_SIZE			32
 #define CFG_H_SIZE			33
 
-/* max string lenghts */
+/* max string lengths */
 #define CFGVAR_LEN		16	/* name */
 #define CFGVALUE_LEN	60	/* string value */
 
@@ -425,24 +461,50 @@ typedef struct {
 
 /********************************************************************/
 
+typedef struct {
+	uid_t uid;
+	const char *login;
+	const char *gecos;
+} USER_ENTRY;
+	
+typedef struct {
+	PANEL_DESC *pd;
+	USER_ENTRY *users;
+	int usr_alloc;			/* allocated entries in 'users' */
+} PANEL_USER;
+
+typedef struct {
+	gid_t gid;
+	const char *group;
+} GROUP_ENTRY;
+	
+typedef struct {
+	PANEL_DESC *pd;
+	GROUP_ENTRY *groups;
+	int grp_alloc;		/* allocated entries in 'groups' */
+} PANEL_GROUP;
+
+/********************************************************************/
+
 /* global variables */
 
 extern const void *pcfg[CFG_VARIABLES];
 
 extern DISPLAY display;
 extern CLEX_DATA clex_data;
-extern TEXTLINE *textline;		/* --> active line */
+extern TEXTLINE *textline;		/* -> active line */
 extern TEXTLINE line_cmd, line_dir, line_tmp;
-extern PANEL_DESC *panel;		/* --> description of the active panel */
+extern PANEL_DESC *panel;		/* -> description of the active panel */
 extern PANEL_FILE *ppanel_file;
 extern PANEL_CFG panel_cfg;
-extern PANEL_BM panel_bm;
+extern PANEL_BM panel_bm_lst, panel_bm_mng;
 extern PANEL_COMPL panel_compl;
 extern PANEL_DIR panel_dir;	
 extern PANEL_DIR_SPLIT panel_dir_split;	
+extern PANEL_GROUP panel_group;
 extern PANEL_HELP panel_help;
 extern PANEL_HIST panel_hist;
-extern PANEL_LOCATE panel_locate;
-extern PANEL_MENU panel_mainmenu, panel_compare;
+extern PANEL_MENU panel_mainmenu, panel_compare, panel_paste;
 extern PANEL_SORT panel_sort;
+extern PANEL_USER panel_user;
 extern CODE next_mode;			/* see control.c comments */
