@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -45,6 +45,7 @@
 #include "cfg.h"		/* config_num() */
 #include "control.h"	/* control_loop() */
 #include "edit.h"		/* edit_update() */
+#include "history.h"	/* get_hist_entry() */
 #include "inout.h"		/* win_waitmsg() */
 #include "list.h"		/* stat2type() */
 #include "sdstring.h"	/* SDSTR() */
@@ -52,17 +53,9 @@
 #include "ustring.h"	/* USTR() */
 #include "util.h"		/* emalloc() */
 
-/*
- * completion types:
- *   any file, directory, executable, command in $PATH, username,
- *   and environment variable
- */
-#define COMPL_TYPE_FILE		0
-#define COMPL_TYPE_DIR		1
-#define COMPL_TYPE_CMD		2
-#define COMPL_TYPE_PATHCMD	3
-#define COMPL_TYPE_USER		4
-#define COMPL_TYPE_ENV		5
+/* internal use only */
+#define COMPL_TYPE_PATHCMD	40	/* search $PATH */
+#define COMPL_TYPE_USERDIR	50	/* with trailing slash (~name/) */
 
 /* environment completion */
 extern char **environ;
@@ -115,6 +108,9 @@ static struct {
 static int cc_alloc = 0;	/* max number of candidates in CC_LIST */
 static int cc_max;			/* max number of candidates to present */
 
+static FLAG done;			/* done: completion not possible or
+							  full (not partial) name was completed */
+
 extern int errno;
 
 static void
@@ -196,22 +192,39 @@ completion_reconfig(void)
 static const char *
 code2string(int type, int uppercase)
 {
+	int i;
+	const char *uc;
+	static char lc[24];
+
 	switch (type) {
 	case COMPL_TYPE_FILE:
-		return uppercase ? "FILE " : "file ";
+		uc = "FILENAME";
+		break;
 	case COMPL_TYPE_DIR:
-		return uppercase ? "DIRECTORY " : "directory ";
+		uc = "DIRECTORY NAME";
+		break;
 	case COMPL_TYPE_PATHCMD:
 	case COMPL_TYPE_CMD:
-		return uppercase ? "COMMAND " : "command ";
+	case COMPL_TYPE_HIST:
+		uc = "COMMAND";
+		break;
 	case COMPL_TYPE_USER:
-		return uppercase ? "USER " : "user ";
+	case COMPL_TYPE_USERDIR:
+		uc = "USER NAME";
+		break;
 	case COMPL_TYPE_ENV:
-		return uppercase ?
-		  "ENVIRONMENT VARIABLE ": "environment variable ";
+		uc = "ENVIRONMENT VARIABLE";
+		break;
 	default:
 		return "";
 	}
+	if (uppercase)
+		return uc;
+
+	for (i = 0; uc[i]; i++)
+		lc[i] = tolower((unsigned char)uc[i]);
+	lc[i] = '\0';
+	return lc;
 }
 
 static int
@@ -234,12 +247,16 @@ void
 compl_prepare(void)
 {
 	panel_compl.pd->cnt = compl.cnt;
-	panel_compl.pd->top = panel_compl.pd->curs =  panel_compl.pd->min;
+	panel_compl.pd->top = panel_compl.pd->min;
+	panel_compl.pd->curs = rq.type == COMPL_TYPE_HIST ? 0 : panel_compl.pd->min;
 	panel_compl.filenames = compl.filenames;
 	panel_compl.description = code2string(rq.type,1);
 
-	qsort(CC_LIST,compl.cnt,sizeof(COMPL_ENTRY),
-	  config_num(CFG_COLLATION) ? qcmp1 : qcmp2);
+	if (rq.type != COMPL_TYPE_HIST)
+		qsort(CC_LIST,compl.cnt,sizeof(COMPL_ENTRY),
+		  config_num(CFG_COLLATION) ? qcmp1 : qcmp2);
+	else
+		win_remark("commands are shown in order of their execution (recent first)");
 
 	panel = panel_compl.pd;
 	/* textline inherited from previous mode */
@@ -293,6 +310,18 @@ complete_environ(void)
 			register_candidate(SDSTR(env_var[i]),0,0,environ[i]);
 }
 
+static void
+complete_history()
+{
+	int i;
+	const HIST_ENTRY *ph;
+
+	for (i = 0; (ph = get_history_entry(i)); i++)
+		if (strncmp(USTR(ph->cmd),rq.str,rq.strlen) == 0)
+			register_candidate(USTR(ph->cmd),0,0,
+			  ph->failed ? "this command failed last time" : 0);
+}
+
 static void
 complete_username(void)
 {
@@ -475,42 +504,51 @@ complete_it(void)
 	rq.str = USTR(dequote_str);
 	rq.strlen = len;
 
-	/* dequote 'dir' + add terminating null byte */
-	us_setsize(&dequote_dir,rq.dirlen + 1);
-	len = dequote_txt(rq.dir,rq.dirlen,USTR(dequote_dir));
-	rq.dir = USTR(dequote_dir);
-	rq.dirlen = len;
-	
+	/* dequote 'dir' + add terminating null byte (not for "." and "/") */
+	if (rq.dirlen != 1 || rq.dir[1] != '\0') {
+		us_setsize(&dequote_dir,rq.dirlen + 1);
+		len = dequote_txt(rq.dir,rq.dirlen,USTR(dequote_dir));
+		rq.dir = USTR(dequote_dir);
+		rq.dirlen = len;
+	}
+
 	compl.cnt = 0;
 	compl.err = 0;
-	compl.quote = textline == &line_cmd;	/* in command line only */
+	compl.quote = 0;
+	compl.filenames = 0;
 	for (i = 0; i < 256; i++)
 		compl.first_char[i] = 0;
-	compl.filenames = 0;
 	if (rq.type == COMPL_TYPE_ENV)
 		complete_environ();
-	else if (rq.type == COMPL_TYPE_USER)
+	else if (rq.type == COMPL_TYPE_USER || rq.type == COMPL_TYPE_USERDIR)
 		complete_username();
-	else if (rq.type == COMPL_TYPE_PATHCMD) {
-		compl.filenames = 1;
-		complete_pathcmd();
-	}
+	else if (rq.type == COMPL_TYPE_HIST)
+		complete_history();
 	else {
-		/* FILE, DIR, CMD completion */
 		compl.filenames = 1;
-		complete_file();
+		if (textline == &line_cmd)
+			compl.quote = 1;	/* in command line only */
+		if (rq.type == COMPL_TYPE_PATHCMD)
+			complete_pathcmd();
+		else
+			/* FILE, DIR, CMD completion */
+			complete_file();
 	}
 }
 
 static void
 insert_candidate(int i)
 {
+	done = 1;
 	edit_nu_insertstr(SDSTR(CC_LIST[i].str) + rq.strlen,compl.quote);
 
 	if ((compl.filenames && IS_FT_DIR(CC_LIST[i].file_type))
-	  || rq.type == COMPL_TYPE_USER)
-		/* ~user is a directory */
+	  || rq.type == COMPL_TYPE_USERDIR /* ~user is a directory */ ) {
 		edit_nu_insertchar('/');
+		done = 0; /* a directory may have subdirectories */
+	}
+	else if (rq.type == COMPL_TYPE_HIST)
+		/* leave it as it is */;
 	else if (USTR(textline->line)[textline->curs] == ' ')
 		textline->curs++;
 	else
@@ -525,6 +563,7 @@ show_results(void)
 	const char *errmsg;
 
 	if (compl.cnt == 0) {
+		done = 1;
 		switch (compl.err) {
 		case 0:
 			errmsg = " (no match)";
@@ -538,7 +577,7 @@ show_results(void)
 		default:
 			errmsg = "";
 		}
-		win_remark_fmt("cannot complete that %sname%s",
+		win_remark_fmt("cannot complete that %s%s",
 		  code2string(rq.type,0),errmsg);
 		return;
 	}
@@ -599,6 +638,7 @@ check_eq_syntax(const char *line, const char *peq)
 /*
  * check backtick (`) syntax:
  * return 1 for opening backtick and 0 for closing backtick
+ * (quoting issues ignored)
  */
 static int
 check_bt_syntax(const char *line, const char *pbt)
@@ -613,6 +653,12 @@ check_bt_syntax(const char *line, const char *pbt)
 	return ok;
 }
 
+static int
+is_sep(int ch)
+{
+	return ch == ' ' || ch == ';' || ch == '&' || ch == '|';
+}
+
 /*
  * compl_file() attempts to complete the partial text in
  * the command line
@@ -624,56 +670,66 @@ check_bt_syntax(const char *line, const char *pbt)
  *       -2 = not the first word (usually one of the arguments)
  */
 int
-compl_file(void)
+compl_file(int type)
 {
-	FLAG first;
+	int first;		/* it is the first word of the command */
 	char ch, *p, *pslash, *pstart, *pend, *plinestart, *plineend;
 
+	done = 0;
 	/*
-	 * plinestart -> start of the command line
-	 * plineend   -> trailing null byte
+	 * plinestart -> start of the input line
+	 * plineend   -> trailing null byte of the input line
 	 * pstart -> start of the string to be completed
-	 * pend   -> immediately after the last character of the string
+	 * pend   -> position immediately after the last character of the string
 	 * pslash -> last slash '/' in string (if any)
 	 */
-	first = 0;
-	plineend = (plinestart = USTR(textline->line)) + textline->size;
-	pstart = pend = plinestart + textline->curs;
-	for (; pstart > plinestart; pstart--) {
-		ch = pstart[-1];
-		if (strchr(" <>",ch))
-			break;
-		if (strchr(";&|",ch)) {
-			first = 1;	/* we have a command separator */
-			break;
-		}
-		if (ch == '=' && check_eq_syntax(plinestart,pstart - 1))
-			break;
-		if (ch == '`') {
-			if (!check_bt_syntax(plinestart,pstart - 1)) {
-				win_remark("cannot complete after `command`");
-				return 0;
+	plinestart = USTR(textline->line);
+	plineend = plinestart + textline->size;
+
+	if (type == COMPL_TYPE_DIRPANEL || type == COMPL_TYPE_HIST) {
+		first = 1;
+		pstart = plinestart;
+		pend = plineend;
+	}
+	else {
+		for (first = -1, pstart = plinestart + textline->curs; first < 0;) {
+			if (pstart == plinestart)
+				first = 1;	/* beginning of line */
+			else if ((ch = pstart[-1]) == ' ') {
+				for (p = pstart; first < 0; p--) {
+					if (p == plinestart)
+						first = 1;
+					else if ((ch = p[-1]) != ' ')
+						first = is_sep((unsigned char)ch);
+				}
 			}
-			first = 1;	/* we have an opening backtick */
-			break;
+			else if (ch == '<' || ch == '>'
+			  || (ch == '=' && check_eq_syntax(plinestart,pstart - 1)))
+				first = 0;	/* I/O redirection or name=value */
+			else if (is_sep((unsigned char)ch))
+				first = 1;	/* after a command separator */
+			else if (ch == '`') {
+				if (!check_bt_syntax(plinestart,pstart - 1)) {
+					win_remark("cannot complete after `command`");
+					done = 1;
+					return 0;
+				}
+				first = 1;	/* inside `command` */
+			}
+			else
+				pstart--;
 		}
+
+		for (pend = plinestart + textline->curs; pend < plineend; pend++)
+			if (is_sep((unsigned char)(ch = *pend)) || ch == '<' || ch == '>')
+				break;
 	}
 
-	while (pend < plineend && *pend != ' ')
-		pend++;
-
-	if (!first)
-		/* check the last non-space character before our string */
-		for (first = 1, p = pstart; p > plinestart;)
-			if ((ch = *--p) != ' ') {
-				if (strchr(";&|",ch) == 0)
-					first = 0;
-				break;
-			}
-
-	if (pstart == pend)
+	if (pstart == pend) {
 		/* nothing to complete */
+		done = 1;
 		return first ? -1 : -2;
+	}
 
 	for (pslash = 0, p = pend; p > pstart;)
 		if (*--p == '/') {
@@ -685,23 +741,15 @@ compl_file(void)
 	textline->curs = pend - plinestart;
 	edit_update_cursor();
 
-	if (pslash == 0) {
+	/* set rq.dir, rq.dirlen, rq.str */
+	if (pslash == 0 || type == COMPL_TYPE_ENV || type == COMPL_TYPE_USER
+	  || type == COMPL_TYPE_USERDIR || type == COMPL_TYPE_HIST ) {
+		rq.str = pstart;
 		rq.dir = ".";
 		rq.dirlen = 1;
-		if (*pstart == '~') {
-			rq.str = pstart + 1;
-			rq.type = COMPL_TYPE_USER;
-		}
-		else if (*pstart == '$') {
-			rq.str = pstart + 1;
-			rq.type = COMPL_TYPE_ENV;
-		}
-		else {
-			rq.str = pstart;
-			rq.type = first ? COMPL_TYPE_PATHCMD : COMPL_TYPE_FILE;
-		}
 	}
 	else {
+		rq.str = pslash + 1;
 		if (*pstart == '~') {
 			*pslash = '\0';
 			rq.dir = dir_tilde(pstart);
@@ -716,78 +764,63 @@ compl_file(void)
 			rq.dir = pstart;
 			rq.dirlen = pslash - pstart;
 		}
-		rq.str = pslash + 1;
-		rq.type = first ? COMPL_TYPE_CMD : COMPL_TYPE_FILE;
 	}
 
+	if (type != COMPL_TYPE_AUTO && type != COMPL_TYPE_DIRPANEL) {
+		/* completion type was set by the user */
+		rq.type = type;
+		if (type == COMPL_TYPE_CMD && pslash == 0)
+			rq.type = COMPL_TYPE_PATHCMD;
+		else if (type == COMPL_TYPE_USER && *rq.str == '~')
+			rq.str++;
+		else if (type == COMPL_TYPE_ENV && *rq.str == '$')
+			rq.str++;
+	}
+	else if (pslash == 0) {
+		if (*pstart == '~') {
+			rq.str++;
+			rq.type = COMPL_TYPE_USERDIR;
+		}
+		else if (type == COMPL_TYPE_DIRPANEL)
+			rq.type = COMPL_TYPE_DIR;
+		else if (*pstart == '$') {
+			rq.str++;
+			rq.type = COMPL_TYPE_ENV;
+		}
+		else
+			rq.type = first ? COMPL_TYPE_PATHCMD : COMPL_TYPE_FILE;
+	}
+	else if (type == COMPL_TYPE_DIRPANEL)
+		rq.type = COMPL_TYPE_DIR;
+	else
+		rq.type = first ? COMPL_TYPE_CMD : COMPL_TYPE_FILE;
+
 	rq.strlen = pend - rq.str;
 	complete_it();
 	show_results();
 	return 0;
 }
 
-/* * compl_dir() attempts to complete a directory name */
-void
-compl_dir(void)
-{
-	char *p, *pslash, *pstart, *pend;
-
-	/*
-	 * pstart -> start of the directory name to be completed
-	 * pslash -> last slash '/' (if any)
-	 * pend   -> trailing null byte
-	 */
-	pstart = USTR(textline->line);
-	pend = pstart + textline->size;
-	for (pslash = 0, p = pend; p > pstart ;)
-		if (*--p == '/') {
-			pslash = p;
-			break;
-		}
-
-	/* move cursor to the end */
-	textline->curs = pend - pstart;
-	edit_update_cursor();
-
-	if (pslash == 0) {
-		rq.dir = ".";
-		rq.dirlen = 1;
-		if (*pstart == '~') {
-			rq.str = pstart + 1;
-			rq.type = COMPL_TYPE_USER;
-		}
-		else {
-			rq.str = pstart;
-			rq.type = COMPL_TYPE_DIR;
-		}
-	}
-	else {
-		if (*pstart == '~') {
-			*pslash = '\0';
-			rq.dir = dir_tilde(pstart);
-			rq.dirlen = strlen(rq.dir);
-			*pslash = '/';
-		}
-		else if (pstart == pslash) {
-			rq.dir = "/";
-			rq.dirlen = 1;
-		}
-		else {
-			rq.dir = pstart;
-			rq.dirlen = pslash - pstart;
-		}
-		rq.str = pslash + 1;
-		rq.type = COMPL_TYPE_DIR;
-	}
-
-	rq.strlen = pend - rq.str;
-	complete_it();
-	show_results();
-}
-
 void
 cx_compl_complete(void)
 {
 	insert_candidate(panel_compl.pd->curs);
 	next_mode = MODE_SPECIAL_RETURN;
 }
+
+static void
+complete_type(int type)
+{
+	if (compl_file(type))
+		win_remark("there is nothing to complete");
+	if (done && get_current_mode() != MODE_FILE)
+		next_mode = MODE_SPECIAL_RETURN;
+}
+
+void cx_complete_auto(void)	{ complete_type(COMPL_TYPE_AUTO);	}
+void cx_complete_file(void)	{ complete_type(COMPL_TYPE_FILE);	}
+void cx_complete_dir(void)	{ complete_type(COMPL_TYPE_DIR);	}
+void cx_complete_cmd(void)	{ complete_type(COMPL_TYPE_CMD);	}
+void cx_complete_user(void)	{ complete_type(COMPL_TYPE_USER);	}
+void cx_complete_env(void)	{ complete_type(COMPL_TYPE_ENV);	}
+void cx_complete_hist(void)	{ complete_type(COMPL_TYPE_HIST);	}
