@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -22,7 +22,7 @@
 #include "directory.h"
 
 #include "cfg.h"			/* config_num() */
-#include "completion.h"		/* compl_dir() */
+#include "completion.h"		/* compl_file() */
 #include "control.h"		/* get_current_mode() */
 #include "edit.h"			/* edit_setprompt() */
 #include "filepanel.h"		/* changedir() */
@@ -48,6 +48,7 @@
 
 /* additional entries to be allocated when the 'dirlist' table is full */
 #define SAVEDIR_ALLOC_UNIT	32
+#define SAVEDIR_ALLOC_MAX	384	/* 'dirlist' size limit */
 
 typedef struct {
 	USTRING dirname;		/* directory name */
@@ -91,7 +92,7 @@ dir_reconfig(void)
 }
 
 /*
- * lenght of the common part of two directory names (FQDNs)
+ * length of the common part of two directory names (FQDNs)
  * (this is not the same as common part of two strings)
  */
 static size_t
@@ -172,11 +173,11 @@ qcmp(const void *e1, const void *e2)
  */
 #define NO_COMPACT	5	/* preserve top 5 directory names */
 void
-dir_main_prepare(void)
+dir_main_panel(void)
 {
 	int i, j, cnt, sub;
 	FLAG store;
-	const char *dirname, *prevdir;
+	const char *dirname, *filter;
 
 	/* D_PANEL_SIZE = AUTO */
 	if (config_num(CFG_D_SIZE) == 0) {
@@ -188,11 +189,14 @@ dir_main_prepare(void)
 		LIMIT_MAX(dp_max,dp_alloc);
 	}
 
-	cnt = 0;
-	for (i = 0; i < dircnt; i++) {
+	filter = panel_dir.pd->filtering ? panel_dir.pd->filter->line : 0;
+
+	for (i = cnt = 0; i < dircnt; i++) {
 		if (cnt == dp_max)
 			break;
 		dirname = USTR(dirlist[i]->dirname);
+		if (filter && !substring(dirname,filter,0))
+			continue;
 		/* compacting */
 		store = 1;
 		if (i >= NO_COMPACT)
@@ -227,17 +231,26 @@ dir_main_prepare(void)
 			common_part(DP_LIST[i].name,DP_LIST[i - 1].name);
 
 	panel_dir.pd->cnt = cnt;
+}
+
+void
+dir_main_prepare(void)
+{
+	int i;
+	const char *prevdir;
+
+	panel_dir.pd->filtering = 0;
+	dir_main_panel();
+	panel_dir.pd->norev = 0;
 	panel_dir.pd->top = panel_dir.pd->min;
 	/* set cursor to previously used directory */
 	panel_dir.pd->curs = 0;
 	prevdir = USTR(dirlist[(dircnt >= 2)]->dirname);	/* [1] or [0] */
-	for (i = 0; i < cnt; i++)
+	for (i = 0; i < panel_dir.pd->cnt; i++)
 		if (DP_LIST[i].name == prevdir) {
 			panel_dir.pd->curs = i;
 			break;
 		}
-	panel_dir.pd->norev = 0;
-
 	panel = panel_dir.pd;
 	textline = &line_dir;
 	edit_nu_kill();
@@ -295,6 +308,7 @@ dir_split_prepare(void)
 	panel_dir_split.pd->norev = 0;
 
 	panel = panel_dir_split.pd;
+
 	/* textline inherited */
 }
 
@@ -309,7 +323,7 @@ filepos_save(void)
 	FLAG new;
 	SAVEDIR *x, *top;
 
-	if (dircnt == diralloc) {
+	if (dircnt == diralloc && diralloc < SAVEDIR_ALLOC_MAX) {
 		diralloc += SAVEDIR_ALLOC_UNIT;
 		dirlist = erealloc(dirlist,diralloc * sizeof(SAVEDIR *));
 		x = emalloc(SAVEDIR_ALLOC_UNIT * sizeof(SAVEDIR));
@@ -320,7 +334,7 @@ filepos_save(void)
 		}
 	}
 
-	for (new = 1, top = dirlist[0], i = 0; i <= dircnt; i++) {
+	for (new = 1, top = dirlist[0], i = 0; i <= dircnt && i < SAVEDIR_ALLOC_MAX; i++) {
 		x = dirlist[i];
 		dirlist[i] = top;
 		top = x;
@@ -387,9 +401,8 @@ filepos_set(void)
 static const char *
 selected_dir(void)
 {
-	return get_current_mode() == MODE_DIR_SPLIT
-	  ? sub_dir(panel_dir_split.pd->curs)
-	  : DP_LIST[panel_dir.pd->curs].name;
+	return get_current_mode() == MODE_DIR_SPLIT ?
+	  sub_dir(panel_dir_split.pd->curs) : DP_LIST[panel_dir.pd->curs].name;
 }
 
 void
@@ -398,7 +411,7 @@ cx_dir_tab(void)
 	const char *dir;
 
 	if (textline->size)
-		compl_dir();
+		compl_file(COMPL_TYPE_DIRPANEL);
 	else if (panel->curs >= 0) {
 		dir = selected_dir();
 		edit_nu_insertstr(dir,0);
@@ -413,21 +426,25 @@ cx_dir_enter(void)
 {
 	const char *dir;
 
-	if (textline->size)
+	if (panel->norev) {
+		/* focus on the input line */
+		if (textline->size == 0)
+			return;
 		dir = dir_tilde(USTR(textline->line));
-	else if (panel->curs < 0) {
-		if (get_current_mode() == MODE_DIR && panel->curs == -1)
-			next_mode = MODE_BM;
-		else
-			next_mode = MODE_SPECIAL_RETURN;
-		return;
 	}
-	else if (get_current_mode() == MODE_DIR_SPLIT)
-		dir = sub_dir(panel_dir_split.pd->curs);
 	else {
-		/* DIR_MAIN -> DIR_SPLIT */
-		next_mode = MODE_DIR_SPLIT;
-		return;
+		/* focus on the panel */
+		if (panel->curs < 0)
+			/* next_mode is set by the EXTRA_LINE table */
+			return;
+		if (get_current_mode() == MODE_DIR_SPLIT)
+			dir = sub_dir(panel_dir_split.pd->curs);
+		else {
+			/* DIR_MAIN -> DIR_SPLIT */
+			cx_edit_kill();
+			next_mode = MODE_DIR_SPLIT;
+			return;
+		}
 	}
 
 	if (changedir(dir) == 0) {
@@ -437,6 +454,6 @@ cx_dir_enter(void)
 
 	if (dir[0] == ' ' || dir[strlen(dir) - 1] == ' ')
 		/* common user error */
-		win_remark("there are spaces before or after "
-		  "the directory name");
+		win_remark("check the spaces around the directory name");
+	next_mode = 0;
 }
