@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -53,7 +53,7 @@
 extern int errno;
 
 #define MAX_SHELL_ARGS	8
-static char *shell_argv[MAX_SHELL_ARGS + 1];
+static char *shell_argv[MAX_SHELL_ARGS + 2 + 1];
 static int cmd_index;		/* which parameter is the command
 							   to be executed */
 
@@ -67,11 +67,23 @@ exec_initialize(void)
 	exec_prompt_reconfig();	/* prompt needs to know the shell */
 }
 
+static int
+shelltype(const char *shell)
+{
+	size_t len;
+
+	shell = base_name(shell);
+	len = strlen(shell);
+	if (len >= 2 && strcmp(shell + len - 2,"sh") == 0)
+		return (len >= 3 && shell[len - 3] == 'c');
+	return 2;
+}
+
 static int
 parse_shellprog(const char *shellprog)
 {
 	char *pch;
-	int i;
+	int i, ac;
 	FLAG arg_end;
 	static USTRING dup = { 0,0 };
 
@@ -92,16 +104,24 @@ parse_shellprog(const char *shellprog)
 			}
 			shell_argv[i++] = pch;
 		}
-	shell_argv[i] = 0;
+	shell_argv[ac = i] = 0;
 
-	while (--i >= 1)
+	for (cmd_index = 0, i = 1; i < ac; i++)
 		if (strcmp(shell_argv[i],"<COMMAND>") == 0) {
 			cmd_index = i;	/* real command belongs here */
-			return 0;		/* success */
+			break;
 		}
 
-	txt_printf("CONFIG: Incorrect SHELLPROG: missing \"<COMMAND>\"\n");
-	return -1;
+	/* append: -c <COMMAND> */
+	if (cmd_index == 0) {
+		shell_argv[ac] = "-c";
+		cmd_index = ac + 1;
+		shell_argv[ac + 2] = 0;
+	}
+
+	clex_data.shelltype = shelltype(shell_argv[0]);
+
+	return 0;
 }
 
 /* prepare to execute commands according to "SHELLPROG" */
@@ -116,16 +136,12 @@ exec_shell_reconfig(void)
 	}
 
 	/* SHELLPROG = AUTO: loginshell -c <COMMAND> */
-	((const char **)shell_argv)[0] = clex_data.shell;
-	shell_argv[1] = "-c";
-	cmd_index = 2;
-	shell_argv[3] = 0;
+	parse_shellprog(clex_data.shell);
 }
 
 void
 exec_prompt_reconfig(void)
 {
-	int len;
 	const char *prompt, *chars;
 	static USTRING cmd_prompt = { 0,0 };
 	static char prompt_char[4] = " ? ";
@@ -135,14 +151,17 @@ exec_prompt_reconfig(void)
 	if (prompt[0] == '\0') {
 		/* PROMPT=AUTO */
 		prompt = base_name(shell_argv[0]);
-		len = strlen(prompt);
-		if (len >= 2 && strcmp(prompt + len - 2,"sh") == 0)
-			if (len >= 3 && prompt[len - 3] == 'c')
-				chars = "%#";	/* looks like C-shell */
-			else
-				chars = "$#";	/* looks like Bourne shell */
-		else
-			chars = ">>";		/* some other shell */
+		switch (clex_data.shelltype) {
+		case 0:
+			chars = "$#";	/* Bourne shell or similar */
+			break;
+		case 1:
+			chars = "%#";	/* C-shell or similar */
+			break;
+		default:
+			chars = ">>";	/* some other shell */
+			break;
+		}
 		prompt_char[1] = chars[clex_data.isroot];
 	}
 	else
@@ -218,7 +237,7 @@ execute(const char *command)
 			puts(
 				"\r\n"
 				"\r\n"
-				"EXEC: CLEX does not support job control\r\n"
+				"EXEC: CLEX does not provide job control\r\n"
 				"EXEC: suspended command will be resumed shortly\r\n"
 				"\r\n");
 			fflush(stdout);
@@ -264,12 +283,10 @@ execute(const char *command)
 	}
 
 	/*
-	 * list directory while the user stares at the screen
-	 * reading the command output, CLEX appears to restart
-	 * very much faster. All output routines called from
-	 * list_directory() must check if curses mode is active,
-	 * win_warning() is the only function designed for this
-	 * situation.
+	 * List the directory while the user stares at the screen
+	 * reading the command output. This way CLEX appears to
+	 * restart much faster. All output routines called from
+	 * list_directory() must check if curses mode is active.
 	 */
 	xterm_title_set(0,command);
 	list_directory();
@@ -388,7 +405,7 @@ check_cd(const char *str)
 
 	tilde = *str == '~';
 	quote = needs_dq = 0;
-	for (real_len = i = 0; (ch = str[i]); i++) {
+	for (real_len = 0, i = tilde /* 0 or 1 */; (ch = str[i]); i++) {
 		if (quote)
 			quote = 0;
 		else if (ch == '\\')
@@ -397,8 +414,8 @@ check_cd(const char *str)
 			if (real_len == 0)
 				real_len = i;
 		}
-		else if (real_len || edit_isspecial(ch))
-			return 0;	/* not a simple 'cd' command */
+		else if (real_len || edit_isspecial((unsigned char)ch))
+			return 0;	/*  not a simple 'cd' command */
 	}
 
 	if (needs_dq) {
@@ -414,7 +431,8 @@ check_cd(const char *str)
 		us_copyn(&dir,str,real_len);
 		dirstr = USTR(dir);
 	}
-	else dirstr = str;
+	else
+		dirstr = str;
 
 	return tilde ? dir_tilde(dirstr) : dirstr;
 }
