@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -23,14 +23,12 @@
 
 #include "cfg.h"			/* config_num() */
 #include "edit.h"			/* edit_update() */
-#include "exec.h"			/* execute_cmd() */
 #include "inout.h"			/* win_remark() */
 #include "panel.h"			/* pan_adjust() */
-#include "undo.h"			/* undo_reset() */
 #include "util.h"			/* emalloc() */
 #include "ustring.h"		/* us_copy() */
 
-#define HS_LIST (panel_hist.hist)
+static HIST_ENTRY **history;/* command line history */
 static int hs_alloc = 0;	/* number of allocated entries */
 static int hs_cnt;			/* entries in use */
 static int pn_index;		/* index for previous/next cmd */
@@ -45,32 +43,65 @@ hist_reconfig(void)
 
 	if (hs_alloc) {
 		free(storage);
-		free(HS_LIST);
+		free(history);
+		free(panel_hist.hist);
+	}
+
+	hs_alloc = config_num(CFG_H_SIZE);
+	storage = emalloc(hs_alloc * sizeof(HIST_ENTRY));
+	history = emalloc(hs_alloc * sizeof(HIST_ENTRY *));
+	panel_hist.hist = emalloc(hs_alloc * sizeof(HIST_ENTRY *));
+	for (i = 0; i < hs_alloc; i++) {
+		history[i] = storage + i;
+		US_INIT(history[i]->cmd);
 	}
 
 	hs_cnt = 0;
-	hs_alloc = config_num(CFG_H_SIZE);
-	storage = emalloc(hs_alloc * sizeof(HIST_ENTRY));
-	HS_LIST = emalloc(hs_alloc * sizeof(HIST_ENTRY *));
-	for (i = 0; i < hs_alloc; i++) {
-		HS_LIST[i] = storage + i;
-		US_INIT(HS_LIST[i]->cmd);
-	}
-
 	hist_reset_index();
 }
 
+void
+hist_panel(void)
+{
+	int i, j;
+	const char *filter;
+	HIST_ENTRY *curs;
+
+	if (VALID_CURSOR(panel_hist.pd))
+		curs = panel_hist.hist[panel_hist.pd->curs];
+	else {
+		curs = 0;
+		panel_hist.pd->curs = 0;
+	}
+	filter = panel_hist.pd->filtering ? panel_hist.pd->filter->line : 0;
+	for (i = j = 0; i < hs_cnt; i++) {
+		if (history[i] == curs)
+			panel_hist.pd->curs = j;
+		if (filter && !substring(USTR(history[i]->cmd),filter,0))
+			continue;
+		panel_hist.hist[j++] = history[i];
+	}
+	panel_hist.pd->cnt = j;
+}
+
 void
 hist_prepare(void)
 {
+	panel_hist.pd->filtering = 0;
+	hist_panel();
 	panel_hist.pd->top = panel_hist.pd->min;
-	panel_hist.pd->cnt = hs_cnt;
-	panel_hist.pd->curs = pn_index >= 0 ? pn_index : 0;
+	panel_hist.pd->curs = pn_index > 0 ? pn_index : 0;
 
 	panel = panel_hist.pd;
 	textline = &line_cmd;
 }
 
+const HIST_ENTRY *
+get_history_entry(int i)
+{
+	return i < hs_cnt ? history[i] : 0;
+}
+
 void
 hist_reset_index(void)
 {
@@ -91,9 +122,9 @@ hist_save(const char *cmd, int failed)
 	hist_reset_index();
 
 	new = 1;
-	for (top = HS_LIST[0], i = 0; i < hs_alloc; i++) {
-		x = HS_LIST[i];
-		HS_LIST[i] = top;
+	for (top = history[0], i = 0; i < hs_alloc; i++) {
+		x = history[i];
+		history[i] = top;
 		top = x;
 		if (i == hs_cnt) {
 			hs_cnt++;
@@ -109,13 +140,15 @@ hist_save(const char *cmd, int failed)
 		us_copy(&top->cmd,cmd);
 	top->failed = failed;
 	
-	HS_LIST[0] = top;
+	history[0] = top;
 }
 
+/* file panel functions */
+
 static void
 warn_fail(int n)
 {
-	if (HS_LIST[n]->failed)
+	if (history[n]->failed)
 		win_remark("this command failed last time");
 }
 
@@ -131,7 +164,7 @@ cx_hist_next(void)
 	if (pn_index-- == 0)
 		edit_putstr(USTR(save_line));
 	else {
-		edit_putstr(USTR(HS_LIST[pn_index]->cmd));
+		edit_putstr(USTR(history[pn_index]->cmd));
 		warn_fail(pn_index);
 	}
 }
@@ -147,81 +180,49 @@ cx_hist_prev(void)
 
 	if (++pn_index == 0)
 		us_xchg(&save_line,&line_cmd.line);
-	edit_putstr(USTR(HS_LIST[pn_index]->cmd));
+	edit_putstr(USTR(history[pn_index]->cmd));
 	warn_fail(pn_index);
 }
 
-void
-cx_hist_complete(void)
-{
-	int i;
-
-	for (i = 0; i < hs_cnt; i++)
-		if (strncmp(USTR(HS_LIST[i]->cmd),USTR(line_cmd.line),
-		  line_cmd.size) == 0) {
-			textline->curs = textline->size;
-			edit_insertstr(USTR(HS_LIST[i]->cmd) + textline->size,0);
-			warn_fail(i);
-			return;
-		}
-	win_remark("cannot complete this command (no match found)");
-}
+/* history panel functions */
 
 void
 cx_hist_paste(void)
 {
-	edit_insertstr(USTR(HS_LIST[panel_hist.pd->curs]->cmd),0);
+	edit_insertstr(USTR(panel_hist.hist[panel_hist.pd->curs]->cmd),0);
 }
 
 void
 cx_hist_enter(void)
 {
-	const char *cmd;
+	if (line_cmd.size == 0)
+		cx_hist_paste();
 
 	next_mode = MODE_SPECIAL_RETURN;
-	if (line_cmd.size)
-		cmd = USTR(line_cmd.line);
-	else if (panel_hist.pd->cnt && panel_hist.pd->curs >= 0)
-		cmd = USTR(HS_LIST[panel_hist.pd->curs]->cmd);
-	else
-		return;
-
-	/*
-	 * Right after the command execution CLEX will return
-	 * from the history mode back to the file mode. This
-	 * little hack prevents one needless screen redrawing.
-	 */
-	panel = ppanel_file->pd;
-
-	if (execute_cmd(cmd)) {
-		cx_edit_kill();
-		undo_reset();
-	}
 }
 
 void
 cx_hist_del(void)
 {
-	int i, del;
-	HIST_ENTRY *x;
+	int i;
+	HIST_ENTRY *del;
+	FLAG move;
 
-	del = panel_hist.pd->curs;
-
-	panel_hist.pd->cnt = --hs_cnt;
-	if (panel_hist.pd->curs == panel_hist.pd->cnt) {
-		panel_hist.pd->curs--;
-		pan_adjust(panel_hist.pd);
+	del = panel_hist.hist[panel_hist.pd->curs];
+	hs_cnt--;
+	for (move = 0, i = 0; i < hs_cnt; i++) {
+		if (history[i] == del) {
+			if (pn_index > i)
+				pn_index--;
+			else if (pn_index == i)
+				hist_reset_index();
+			move = 1;
+		}
+		if (move)
+			history[i] = history[i + 1];
 	}
-
-	x = HS_LIST[del];
-	for (i = del; i < hs_cnt; i++)
-		HS_LIST[i] = HS_LIST[i + 1];
-	HS_LIST[hs_cnt] = x;
-
-	if (pn_index > del)
-		pn_index--;
-	else if (pn_index == del)
-		hist_reset_index();
-
+	history[hs_cnt] = del;
+	hist_panel();
+	pan_adjust(panel_hist.pd);
 	win_panel();
 }
