@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -145,6 +145,7 @@ screen_draw_all(void)
 		pan_adjust(panel);
 		win_panel();
 		win_info();
+		win_filter();
 	}
 	edit_adjust();
 	win_edit();
@@ -219,14 +220,18 @@ screen_refresh(void)
 	if (pos_wait || pos_resize || pos_panel)
 		win_position();		/* display/clear message */
 
-	if (textline == 0)
-		pos = 0;
-	else if (textline->offset)
-		pos = 1 + textline->curs - textline->offset;
-	else
-		pos = textline->curs + textline->promptlen;
-	move(display.panlines + 5 + pos / display.scrcols,
-	  pos % display.scrcols);
+	if (panel->filtering == 1)
+		move(display.panlines + 2,13 + panel->filter->curs);
+	else {
+		if (textline == 0)
+			pos = 0;
+		else if (textline->offset)
+			pos = 1 + textline->curs - textline->offset;
+		else
+			pos = textline->curs + textline->promptlen;
+		move(display.panlines + 5 + pos / display.scrcols,
+		  pos % display.scrcols);
+	}
 	refresh();
 }
 
@@ -283,10 +288,18 @@ kbd_getraw(void)
 #endif
 #ifdef KEY_SUSPEND
 		if (key == KEY_SUSPEND)
-			key = CH_CTRL('Z');
+			return CH_CTRL('Z');
 #endif
 		if (key == KEY_BACKSPACE)
-			key = CH_CTRL('H');
+			return CH_CTRL('H');
+		if (key == KEY_ENTER)
+			return CH_CTRL('M');
+		if (key == CH_CTRL('G'))
+			return CH_CTRL('C');
+		if (key == KEY_DC)
+			return '\177';
+		if (key == KEY_IL)
+			return KEY_IC;
 		return key;
 	}
 }
@@ -316,7 +329,7 @@ kbd_input(void)
 
 	/* show ASCII code */
 	if (info_remark == 0 && textline != 0) {
-		ch = USTR(textline->line)[textline->curs] & 0xFF;
+		ch = (unsigned char)USTR(textline->line)[textline->curs];
 		if (ch != '\0' && iscntrl(ch)) {
 			if (ch >= 1 && ch <= 26)
 				win_remark_fmt("Ctrl-%c  ASCII %d",ch + 'A' - 1,ch);
@@ -359,7 +372,8 @@ kbd_esc(void)
  * win_panel							 DIR bin
  * win_panel							>DIR etc <
  * win_panel							 DIR man
- * win_frame, win_waitmsg, win_position	---------< 3/10 >-
+ * win_frame, win_filter,
+ * win_waitmsg, win_position			-< filt >----< 3/9 >-
  * win_info, win_warning, win_remark	0755 rwxr-xr-x
  * win_bar								[ CLEX file manager ]
  * win_edit								shell $ ls -l_
@@ -491,6 +505,27 @@ win_waitmsg(void)
 	}
 }
 
+void
+win_filter(void)
+{
+	int len;
+
+	len = INPUT_STR + 12;
+	move(display.panlines + 2,2);
+	if (panel->filtering) {
+		if (panel->type == PANEL_TYPE_FILE && ppanel_file->filtype)
+			addstr("< pattern: ");	/* 11 */
+		else
+			addstr("<  filter: ");	/* 11 */
+		attrset(attrb);
+		addstr(panel->filter->line);
+		attrset(A_NORMAL);
+		addstr(" >");			/* 2 */
+		len -= 13 + panel->filter->size;
+	}
+	char_line(framechar,len);
+}
+
 /*
  * putstr_trunc() writes string 'str' padding or truncating it
  * to the total length of 'maxwidth'. Its behavior might be altered
@@ -525,7 +560,7 @@ putstr_trunc(const char *str, int maxwidth, int options)
 	}
 
 	for (i = 0; i < maxwidth - 1; i++)
-		if ( (ch = str[i] & 0xFF) )
+		if ( (ch = (unsigned char)str[i]) )
 			ADDCH(ch);
 		else {
 			/* EOL */
@@ -535,7 +570,7 @@ putstr_trunc(const char *str, int maxwidth, int options)
 			return maxwidth;
 		}
 
-	ch = str[i] & 0xFF;
+	ch = (unsigned char)str[i];
 	if (ch == '\0') {
 		if (options & OPT_NOPAD)
 			return i;
@@ -603,19 +638,17 @@ win_heading(void)
 	addch(' ');
 	len = 1;
 	switch (mode) {
-	case MODE_BM:
-		msg = "CHANGE WORKING DIRECTORY > BOOKMARKS";
-		break;
-	case MODE_BM_MANAGER:
-		msg = "BOOKMARK MANAGER   "
-		  "[ U/D = up/down, N = new, <del> = delete, <enter> = edit ]";
-		break;
 	case MODE_BM_EDIT:
 		msg = "BOOKMARK MANAGER > EDIT";
 		break;
+	case MODE_BM_LIST:
+		msg = "CHANGE WORKING DIRECTORY > BOOKMARKS";
+		break;
+	case MODE_BM_MANAGER:
+		msg = "BOOKMARK MANAGER  |  U/D = up/down, <insert> or I, <del> or R, <enter> = edit";
+		break;
 	case MODE_CFG:
-		msg = "CONFIGURATION    "
-		  "[ <enter> = change, O = original, S = standard ]";
+		msg = "CONFIGURATION  |  <enter> = change, O = original, S = standard";
 		break;
 	case MODE_CFG_EDIT_NUM:
 	case MODE_CFG_EDIT_TXT:
@@ -627,13 +660,16 @@ win_heading(void)
 	case MODE_COMPL:
 		msg = panel_compl.description;
 		addstr(msg);
-		len = 1 + strlen(msg);
-		msg = "COMPLETION";
+		len += strlen(msg);
+		msg = " COMPLETION";
 		break;
 	case MODE_DIR:
 	case MODE_DIR_SPLIT:
 		msg = "CHANGE WORKING DIRECTORY";
 		break;
+	case MODE_GROUP:
+		msg = "GROUP INFORMATION";
+		break;
 	case MODE_HELP:
 		addstr("HELP: ");
 		len = 7;
@@ -641,16 +677,14 @@ win_heading(void)
 		msg = panel_help.heading;
 		break;
 	case MODE_HIST:
-		msg = "COMMAND HISTORY    "
-		  "[ <tab> = insert, <enter> = execute, <del> = delete ]";
-		break;
-	case MODE_P_LOCATE:
-	case MODE_S_LOCATE:
-		msg = "LOCATE FILE";
+		msg = "COMMAND HISTORY  |  <tab> = insert, <esc> <del> = delete";
 		break;
 	case MODE_MAINMENU:
 		msg = "MAIN FUNCTION MENU";
 		break;
+	case MODE_PASTE:
+		msg = "COMPLETE/INSERT NAME";
+		break;
 	case MODE_SELECT:
 	case MODE_DESELECT:
 		msg = "DESELECT FILES";
@@ -660,6 +694,9 @@ win_heading(void)
 	case MODE_SORT:
 		msg = "SORT ORDER";
 		break;
+	case MODE_USER:
+		msg = "USER INFORMATION";
+		break;
 	default:
 		/* mode not set yet (during start-up) */
 		msg = "CLEX";
@@ -693,12 +730,12 @@ print_perms(const char *octal)
 static int
 print_fields(FILE_ENTRY *pfe, int width, const char *fields)
 {
-	char ch;
-	FLAG fld, left_align;
 	const char *txt;
-	int i, fw;
+	FLAG fld, left_align;
+	int ch, i, fw;
+	static char field[3] ="$x";
 
-	for (fld = left_align = 0; width > 0 && (ch = *fields++); ) {
+	for (fld = left_align = 0; width > 0 && (ch = (unsigned char)*fields++); ) {
 		if (!TCLR(fld)) {
 			if (ch == '$')
 				fld = 1;
@@ -747,10 +784,18 @@ print_fields(FILE_ENTRY *pfe, int width, const char *fields)
 				fw = 9;	/* rwxrwxrwx */
 				txt = 0;
 				break;
+			case 'P':	/* permissions (alternative format) */
+				fw = 9;
+				txt = pfe->normal_mode ? "" : 0;
+				break;
 			case 's':	/* file size (device major/minor) */
 				fw = FE_SIZE_DEV_STR - 1;
 				txt = pfe->size_str;
 				break;
+			case 'S':	/* file size (not for directories) */
+				fw = FE_SIZE_DEV_STR - 1;
+				txt = IS_FT_DIR(pfe->file_type) ? "" : pfe->size_str;
+				break;
 			case 't':	/* file type */
 				fw = 4;
 				txt = type_symbol[pfe->file_type];
@@ -772,8 +817,9 @@ print_fields(FILE_ENTRY *pfe, int width, const char *fields)
 				txt = "|";
 				break;
 			default:	/* syntax error */
-				fw = 1;
-				txt = "?";
+				fw = 2;
+				field[1] = ch;
+				txt = field;
 			}
 
 			if (width < fw)
@@ -814,34 +860,19 @@ static void
 win_info_extra_line(int ln)
 {
 	const char *msg;
-	static const char *infoline_cfg[] = {
-		"  No configuration change",
-		"  Use the new configuration in this session",
-		"  Save the configuration to ~/.clexrc"
-	},
-	*infoline_cfg_admin[] = {
-		"  Exit program",
-		"  Save the new system-wide defaults to " CONFIG_FILE
-	};
 
-	msg = 0;
-	switch (panel->type) {
-	case PANEL_TYPE_CFG:
-		msg = infoline_cfg[ln];
-		break;
-	case PANEL_TYPE_CFG_ADMIN:
-		msg = infoline_cfg_admin[ln];
-		break;
-	case PANEL_TYPE_BM:
-		if (ln == 2) {
+	msg = panel->extra[ln].info;
+	if (msg) {
+		if (msg[0] == '@') {
+			/* special handling */
 			addstr("  working directory is "); /* 23 */
 			putstr_trunc(USTR(ppanel_file->dir),display.scrcols - 23,0);
-			return;
 		}
-		break;
+		else {
+			addstr("  ");
+			putstr_trunc(msg,display.scrcols - 2,0);
+		}
 	}
-	if (msg)
-		putstr_trunc(msg,display.scrcols,0);
 	else
 		clrtoeol();
 }
@@ -902,22 +933,18 @@ win_info(void)
 
 	switch (panel->type) {
 		case PANEL_TYPE_CFG:
-		case PANEL_TYPE_CFG_ADMIN:
 			addstr("  ");					/* 2 */
 			msg = panel_cfg.config[panel->curs].help;
 			putstr_trunc(msg,display.scrcols - 2,0);
 			break;
 		case PANEL_TYPE_COMPL:
 			if ( (msg = panel_compl.candidate[panel->curs].aux) ) {
-				addstr("  location: ");		/* 12 */
-				putstr_trunc(msg,display.scrcols - 12,0);
+				addstr("  additional data: ");		/* 19 */
+				putstr_trunc(msg,display.scrcols - 19,0);
 			}
 			else
 				clrtoeol();
 			break;
-		case PANEL_TYPE_LOCATE:
-			pfe_info(panel_locate.files[panel->curs]);
-			break;
 		case PANEL_TYPE_FILE:
 			pfe_info(ppanel_file->files[panel->curs]);
 			break;
@@ -973,8 +1000,7 @@ win_remark_fmt(const char *format, ...)
 	static char buff[160];
 
 	if (!display.curses) {
-		/* this should not happen */
-		txt_printf("BUG: win_remark(\"%s\")\n"
+		txt_printf("BUG: win_remark_fmt(\"%s\",...)\n"
 		  "     called in text mode\n",format);
 		return;
 	}
@@ -991,8 +1017,8 @@ void
 win_remark(const char *str)
 {
 	if (!display.curses) {
-		/* this should not happen */
-		win_remark_fmt(str);
+		txt_printf("BUG: win_remark(\"%s\")\n"
+		  "     called in text mode\n",str);
 		return;
 	}
 
@@ -1035,7 +1061,7 @@ win_bar(void)
 	len = strlen(clex_data.login_at_host) + 2;	/* + 2 spaces */
 	addstr(" CLEX file manager - ");			/* 21 */
 	putstr_trunc(clex_data.admin ? "ADMIN MODE" :
-	  "press ctrl-G for main menu",display.scrcols - 21 - len,0);
+	  "alt-M for menu, F1 for help",display.scrcols - 21 - len,0);
 	addch(' ');
 	putstr_trunc(clex_data.login_at_host,len + 1,0);
 	attrset(A_NORMAL);
@@ -1063,13 +1089,16 @@ win_edit(void)
 
 	/* prompt (or continuation mark) */
 	offset = textline->offset;
-	attrset(attrb);
 	width = display.textline_area;
 	if (offset == 0) {
+		if ((panel->type != PANEL_TYPE_DIR_SPLIT
+		  && panel->type != PANEL_TYPE_DIR) || panel->norev)
+		attrset(attrb);
 		addstr(textline->prompt);
 		width -= textline->promptlen;
 	}
 	else {
+		attrset(attrb);
 		addch('<');
 		width--;
 	}
@@ -1079,32 +1108,10 @@ win_edit(void)
 
 /****** win_panel() and friends  ******/
 
-static void
-draw_line_pfe(FILE_ENTRY *pfe)
-{
-	int width;
-
-	if (pfe->select)
-		attron(attrb);
-
-	/* 10 columns reserved for the filename */
-	width = 10 + print_fields(pfe,display.pancols - 10,layout_panel);
-	if (!pfe->symlink)
-		putstr_trunc(SDSTR(pfe->file),width,0);
-	else {
-		width -= putstr_trunc(SDSTR(pfe->file),width,OPT_NOPAD);
-		width -= putstr_trunc(" -> ",width,OPT_NOPAD);
-		putstr_trunc(USTR(pfe->link),width,0);
-	}
-
-	if (pfe->select)
-		attroff(attrb);
-}
-
 static void
 draw_line_bm(int ln)
 {
-	putstr_trunc(USTR(panel_bm.bm[ln]),display.pancols,0);
+	putstr_trunc(USTR(panel_bm_lst.bm[ln]),display.pancols,0);
 }
 
 static void
@@ -1119,11 +1126,12 @@ static void
 draw_line_compare(int ln)
 {
 	static const char *description[] = {
-		"0: name, size, type (symbolic links ok)",
-		"1: name, size, type",
-		"2: name, size, type, ownership+permissions",
-		"3: name, size, type, contents",
-		"4: name, size, type, ownership+permissions, contents",
+		"0: name, type       (symbolic links ok)",
+		"1: name, type, size (symbolic links ok)",
+		"2: name, type, size",
+		"3: name, type, size, ownership+permissions",
+		"4: name, type, size, contents",
+		"5: name, type, size, ownership+permissions, contents",
 	};
 
 	putstr_trunc(description[ln],display.pancols,0);
@@ -1178,7 +1186,32 @@ draw_line_dir_split(int ln)
 static void
 draw_line_file(int ln)
 {
-	draw_line_pfe(ppanel_file->files[ln]);
+	FILE_ENTRY *pfe;
+	int width;
+
+	pfe = ppanel_file->files[ln];
+	if (pfe->select)
+		attron(attrb);
+
+	/* 10 columns reserved for the filename */
+	width = 10 + print_fields(pfe,display.pancols - 10,layout_panel);
+	if (!pfe->symlink)
+		putstr_trunc(SDSTR(pfe->file),width,0);
+	else {
+		width -= putstr_trunc(SDSTR(pfe->file),width,OPT_NOPAD);
+		width -= putstr_trunc(" -> ",width,OPT_NOPAD);
+		putstr_trunc(USTR(pfe->link),width,0);
+	}
+
+	if (pfe->select)
+		attroff(attrb);
+}
+
+static void
+draw_line_grp(int ln)
+{
+	printw("%10u  ",(unsigned int)panel_group.groups[ln].gid);
+	putstr_trunc(panel_group.groups[ln].group,display.pancols - 12,0);
 }
 
 static void
@@ -1203,42 +1236,58 @@ draw_line_hist(int ln)
 	putstr_trunc(USTR(panel_hist.hist[ln]->cmd),display.pancols - 8,0);
 }
 
-static void
-draw_line_locate(int ln)
-{
-	draw_line_pfe(panel_locate.files[ln]);
-}
-
 static void
 draw_line_mainmenu(int ln)
 {
 	static const char *description[] = {
-		"help                                     <F1> ",
-		"change working directory                 <esc> D",
-		"  change into root directory             <esc> /",
-		"  change into parent directory           <esc> .",
-		"  change into home directory             <esc> ~",
-		"  bookmarks                              <esc> B",
-		"command history                          <esc> H",
-		"sort order for filenames                 <esc> S",
+		"help                                     <F1>",
+		"change working directory                 alt-W",
+		"  change into root directory             alt-/",
+		"  change into parent directory           alt-.",
+		"  change into home directory             alt-~",
+		"  bookmarks                              alt-K",
+		"command history                          alt-H",
+		"sort order for filenames                 alt-S",
 		"re-read current directory                ctrl-R",
-		"locate file (substring)                  ctrl-F",
-		"locate file (pattern)                    <esc> ctrl-F",
-		"compare directories:                     <esc> =",
-		"select files:  select all                ctrl-G +",
-		"               deselect all              ctrl-G -",
-		"               select using pattern      <esc> +",
-		"               deselect using pattern    <esc> -",
-		"               invert selection          <esc> *",
-		"configure CLEX                           <esc> C",
-		"program version                          <esc> V",
-		"quit                                     <esc> Q"
+		"compare directories                      alt-=",
+		"filter on/off                            ctrl-F",
+		"user (group) information                 alt-U (alt-G)",
+		"select files:  select all                (alt-M) +",
+		"               deselect all              (alt-M) -",
+		"               select using pattern      alt-+",
+		"               deselect using pattern    alt--",
+		"               invert selection          alt-*",
+		"configure CLEX                           alt-C",
+		"program version                          alt-V",
+		"quit                                     alt-Q"
 		/* must correspond with tab_mainmenu[] in control.c */
 	};
 
 	putstr_trunc(description[ln],display.pancols,0);
 }
 
+static void
+draw_line_pastemenu(int ln)
+{
+	static const char *description[] = {
+		"complete name (auto)",
+		"complete filename - any type",
+		"complete filename - directory",
+		"complete filename - executable",
+		"complete username",
+		"complete environment variable name",
+		"complete command from the command history         alt-P",
+		"insert current filename                           <F2>",
+		"insert all selected filenames               <esc> <F2>",
+		"insert full pathname of current file              ctrl-A",
+		"insert secondary working directory name           ctrl-E",
+		"insert current working directory name       <esc> ctrl-E",
+		"insert the target of a symbolic link              ctrl-O"
+		/* must correspond with tab_pastemenu[] in control.c */
+	};
+	putstr_trunc(description[ln],display.pancols,0);
+}
+
 static void
 draw_line_sort(int ln)
 {
@@ -1247,46 +1296,49 @@ draw_line_sort(int ln)
 		".extension",
 		"size [small -> large]",
 		"size [large -> small]",
-		"time [recent -> old]",
-		"time [old -> recent]",
+		"time of last modification [recent -> old]",
+		"time of last modification [old -> recent]",
 		"reversed name (use in sendmail queue directory)"
 		/* must correspond with SORT_XXX */
 	};
 
-	addstr(panel_sort.order == ln ? "(x)" : "( )");	/* 3 */
-	addstr(" sort by ");							/* 9 */
+	addstr(panel_sort.order == ln ? "(x" : "( ");	/* 2 */
+	addstr(") sort by ");							/* 10 */
 	putstr_trunc(description[ln],display.pancols - 12,0);
 }
 
+static void
+draw_line_usr(int ln)
+{
+	int len;
+
+	printw("%10u  ",(unsigned int)panel_user.users[ln].uid);
+	len = putstr_trunc(panel_user.users[ln].login,48,OPT_NOPAD);
+	if (len < 14) {
+		BLANK(14 - len);
+		len = 14;
+	}
+	addstr(" ");
+	putstr_trunc(panel_user.users[ln].gecos,display.pancols - len - 13,0);
+}
+
 static void
 draw_panel_extra_line(int ln)
 {
 	const char *msg;
-	static const char
-	  *menu_cfg[]        = { "Cancel", "Accept", "Accept+Save" },
-	  *menu_cfg_admin[]  = { "Exit", "Save" },
-	  *menu_bm[]         = { 0, "Manage bookmarks",
-							 "Bookmark current working directory" },
-	  *menu_dir[]        = { 0, "Bookmarks" };
+	FLAG help;
 
-	addstr("--> ");			/* 4 */
-	switch (panel->type) {
-		case PANEL_TYPE_BM:
-			msg = menu_bm[ln];
-			break;
-		case PANEL_TYPE_CFG:
-			msg = menu_cfg[ln];
-			break;
-		case PANEL_TYPE_CFG_ADMIN:
-			msg = menu_cfg_admin[ln];
-			break;
-		case PANEL_TYPE_DIR:
-			msg = menu_dir[ln];
-			break;
-		default:
-			msg = 0;
+	help = panel->type == PANEL_TYPE_HELP;
+	if (help) {
+		attrset(attrb);
+		addstr("==> ");			/* 4 */
 	}
+	else
+		addstr("--> ");			/* 4 */
+	msg = panel->extra[ln].text;
 	putstr_trunc(msg ? msg : "Leave this panel",display.pancols - 4,0);
+	if (help)
+		attrset(A_NORMAL);
 }
 
 static void
@@ -1295,10 +1347,10 @@ draw_panel_line(int y)
 	int ln;
 	/* must correspond with PANEL_TYPE_XXX */
 	static void (*draw_line[])(int) = {
-	  draw_line_bm, draw_line_bm, draw_line_cfg, draw_line_cfg,
-	  draw_line_compare, draw_line_compl, draw_line_dir,
-	  draw_line_dir_split, draw_line_file, draw_line_help, draw_line_hist,
-	  draw_line_locate, draw_line_mainmenu, draw_line_sort
+	  draw_line_bm, draw_line_cfg, draw_line_compare, draw_line_compl,
+	  draw_line_dir, draw_line_dir_split, draw_line_file, draw_line_grp,
+	  draw_line_help, draw_line_hist, draw_line_mainmenu,
+	  draw_line_pastemenu, draw_line_sort, draw_line_usr
 	};
 
 	move(2 + y,0);
