@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -85,12 +85,12 @@ lang_initialize(void)
 			} else if (strncmp(in,"70",2) == 0) {
 				in += 2;
 				*out++ = 'y';
-			} else if (isalpha(*in & 0xFF)) {
+			} else if (isalpha((unsigned char)*in)) {
 				/*
 				 * assuming this is month in written form
 				 * e.g. feb, February, or even some other language
 				 */
-				while (isalpha(*++in & 0xFF))
+				while (isalpha((unsigned char)*++in))
 					;
 				*out++ = 'M';
 			} else {
