@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -17,6 +17,7 @@
 #include <sys/types.h>		/* time_t */
 #include <sys/stat.h>		/* stat() */
 #include <errno.h>			/* errno */
+#include <filter.h>			/* filter_update() */
 #include <stdio.h>			/* sprintf() */
 #include <string.h>			/* strcmp() */
 #include <unistd.h>			/* stat() */
@@ -80,17 +81,20 @@
  */
 #define FE_ALLOC_UNIT	128
 
+#define CACHE_SIZE 		24	/* size of cache for user/group name lookups */
+
 extern int errno;
 
 static time_t now;
-static FLAG do_a, do_d, do_i, do_l, do_L, do_m, do_M, do_o, do_p, do_s;
+static FLAG do_a, do_d, do_i, do_l, do_L, do_m, do_M, do_o, do_s;
 static FLAG clock24, use_pathname = 0;
 static char sep000;			/* thousands separator */
-static int tpad, dpad;		/* lenght of padding for time and date */
+static int tpad, dpad;		/* length of padding for time and date */
 static int K2;				/* kilobyte/2 */
 static const char *fmt_date;/* format string for date */
 static dev_t dirdev;		/* data of the inspected directory */
 static mode_t normal_file, normal_dir;
+static int ucache_cnt = 0, gcache_cnt = 0;
 
 void
 list_reconfig(void)
@@ -107,7 +111,7 @@ list_reconfig(void)
 	fmt_date = config_str(CFG_FMT_DATE);
 	if (*fmt_date == '\0')
 		fmt_date = lang_fmt_date();
-	/* compute the field lenght */
+	/* compute the field length */
 	for (dlen = i = 0; (ch = fmt_date[i]); i++)
 		switch (ch) {
 		case 'Y': dlen += 4; break;
@@ -141,7 +145,7 @@ list_reconfig(void)
 
 	/*
 	 * time string and date string are displayed in the same column,
-	 * prepare padding string to equalize their lenghts
+	 * prepare padding string to equalize their lengths
 	 */
 	if (tlen >= dlen) {
 		tpad = 0;
@@ -168,8 +172,8 @@ list_reconfig(void)
 	}
 
 	/* which fields are going to be displayed ? */
-	do_a = do_d = do_i = do_l = do_L = 0;
-	do_m = do_M = do_o = do_p = do_s = 0;
+	do_a = do_d = do_i = do_l = 0;
+	do_L = do_m = do_M = do_o = do_s = 0;
 	for (fld = 0, fields = config_layout; (ch = *fields++); )
 		if (!TCLR(fld)) {
 			if (ch == '$')
@@ -182,10 +186,12 @@ list_reconfig(void)
 			case 'i': do_i = 1; break;
 			case 'L': do_L = 1; break;
 			case 'l': do_l = 1; break;
-			case 'M': do_M = do_m = 1; break;
+			case 'P':					/* $P -> $M */
+			case 'M': do_M = 1; 		/* $M -> $m */
+			case 'p':					/* $p -> $m */
 			case 'm': do_m = 1; break;
 			case 'o': do_o = 1; break;
-			case 'p': do_p = do_m = 1; break;
+			case 'S':					/* $S -> $s */
 			case 's': do_s = 1; break;
 			/* default:   ignore unknown formatting character */
 			}
@@ -204,7 +210,7 @@ list_initialize(void)
 /*
  * when calling any of the stat2xxx() functions always make sure
  * the string provided as 'str' argument can store the result.
- * See FE_XXX_STR #defines in clex.h for proper sizes.
+ * Check the FE_XXX_STR #defines in clex.h for proper sizes.
  */
 
 static void
@@ -399,21 +405,85 @@ stat2type(mode_t mode, uid_t uid)
 	return FT_OTHER;
 }
 
+static void
+id2name(char *str, int leftalign, const char *name, unsigned int id)
+{
+	char number[16];
+	size_t len;
+
+	if (name == 0) {
+		sprintf(number,"%u",id);
+		name = number;
+	}
+	len = strlen(name);
+	sprintf(str,len <= 9 ? (leftalign ? "%-9s" : "%9s") : "%4.4s>%4s",name,name + len - 4);
+
+}
+
+static const char *
+uid2name(uid_t uid)
+{
+	static int pos = 0, replace = 0;
+	static struct {
+		uid_t uid;
+		char name[FE_NAME_STR];
+	} cache[CACHE_SIZE];
+
+	if (pos < ucache_cnt && uid == cache[pos].uid)
+		return cache[pos].name;
+
+	for (pos = 0; pos < ucache_cnt; pos++)
+		if (uid == cache[pos].uid)
+			return cache[pos].name;
+
+	if (ucache_cnt < CACHE_SIZE)
+		pos = ucache_cnt++;
+	else {
+		pos = replace;
+		if (++replace >= CACHE_SIZE)
+			replace = 0;
+	}
+	cache[pos].uid = uid;
+	id2name(cache[pos].name,0,lookup_login(uid),(unsigned int)uid);
+
+	return cache[pos].name;
+}
+
+static const char *
+gid2name(gid_t gid)
+{
+	static int pos = 0, replace = 0;
+	static struct {
+		gid_t gid;
+		char name[FE_NAME_STR];
+	} cache[CACHE_SIZE];
+
+	if (pos < gcache_cnt && gid == cache[pos].gid)
+		return cache[pos].name;
+
+	for (pos = 0; pos < gcache_cnt; pos++)
+		if (gid == cache[pos].gid)
+			return cache[pos].name;
+
+	if (gcache_cnt < CACHE_SIZE)
+		pos = gcache_cnt++;
+	else {
+		pos = replace;
+		if (++replace >= CACHE_SIZE)
+			replace = 0;
+	}
+	cache[pos].gid = gid;
+	id2name(cache[pos].name,1,lookup_group(gid),(unsigned int)gid);
+
+	return cache[pos].name;
+}
+
 static void
 stat2owner(char *str, uid_t uid, gid_t gid)
 {
-	char number1[16], number2[16];
-	const char *pch1, *pch2;
-
-	if ( (pch1 = lookup_login(uid)) == 0) {
-		sprintf(number1,"%u",(unsigned int)uid);
-		pch1 = number1;
-	}
-	if ( (pch2 = lookup_group(gid)) == 0) {
-		sprintf(number2,"%u",(unsigned int)gid);
-		pch2 = number2;
-	}
-	sprintf(str,"%8.8s:%-8.8s",pch1,pch2);
+		strcpy(str,uid2name(uid));
+		str[FE_NAME_STR - 1] = ':';
+		strcpy(str + FE_NAME_STR,gid2name(gid));
 }
 
 static void
@@ -573,7 +643,7 @@ directory_read(void)
 
 	/*
 	 * step #1: process selected files already listed in the panel
-	 * in order not to loose their selection mark
+	 * in order not to lose their selection mark
 	 */
 	cnt1 = 0;
 	for (i = 0; cnt1 < ppanel_file->selected; i++) {
@@ -644,12 +714,24 @@ directory_read(void)
 	closedir(dd);
 }
 
+/* directory read wrapper */
 static void
 filepanel_read(void)
 {
 	filepos_save();
+	if (ppanel_file->pd->filtering) {
+		/* suspend filtering */
+		ppanel_file->pd->cnt = ppanel_file->filt_cnt;
+		ppanel_file->selected += ppanel_file->filt_sel;
+	}
 	directory_read();
-	sort_files();
+	if (ppanel_file->pd->filtering) {
+		/* resume filtering */
+		ppanel_file->filt_cnt = ppanel_file->pd->cnt;
+		ppanel_file->pd->filter->changed = 1;
+	}
+	else
+		sort_files();
 	filepos_set();
 	ppanel_file->expired = 0;
 }
@@ -660,8 +742,10 @@ list_directory(void)
 	now = time(0);
 
 	/* password data change invalidates data in both panels */
-	if (userdata_refresh())
+	if (userdata_refresh()) {
 		ppanel_file->other->expired = 1;
+		ucache_cnt = gcache_cnt = 0;
+	}
 
 	filepanel_read();
 }
@@ -669,20 +753,33 @@ list_directory(void)
 void
 list_both_directories(void)
 {
+	PANEL_DESC *savep;
+
+	savep = panel;
+
 	now = time(0);
-	userdata_refresh();
+	if (userdata_refresh())
+		ucache_cnt = gcache_cnt = 0;
 
 	filepanel_read();
+	panel = ppanel_file->pd;
+	if (panel->filtering)
+		filter_update();
 
 	/*
 	 * warning: during the re-read of the secondary panel,
 	 * the primary panel does not correspond with the current
-	 * working directory; this is the only exception
+	 * working directory
 	 */
-	panel = (ppanel_file = ppanel_file->other)->pd;
+	ppanel_file = ppanel_file->other;
 	pathname_set_directory(USTR(ppanel_file->dir));
 	use_pathname = 1;	/* must prepend directory name */
 	filepanel_read();
+	panel = ppanel_file->pd;
+	if (panel->filtering)
+		filter_update();
 	use_pathname = 0;
-	panel = (ppanel_file = ppanel_file->other)->pd;
+	ppanel_file = ppanel_file->other;
+
+	panel = savep;
 }
