@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -203,7 +203,7 @@ parse_expression(void)
 		case STATE_DOUBLE:
 			if (ch == '\\')
 				/* backslash quotes only \ " and $ */
-				if (strchr("\\\"$",expr[i + 1]))
+				if (strchr("\\\"$",(unsigned char)expr[i + 1]))
 					TRANS(IGNORE,DOUBLE_BACKSLASH);
 				else
 					SYM(LITERAL);
