@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -45,6 +45,8 @@ select_prepare(void)
 	const char *prompt;
 
 	panel = ppanel_file->pd;
+	if (panel->filtering == 1)
+		panel->filtering = 2;
 
 	prompt = "DESELECT files: ";
 	if (get_current_mode() == MODE_SELECT)
@@ -214,11 +216,12 @@ file_cmp(const char *file1, const char *file2)
 /*
  * compare panels
  * levels:
- *	0: name, size, type (symbolic links ok)
- *	1: name, size, type (exactly)
- *	2: name, size, type, ownership&mode
- *	3: name, size, type, contents
- *	4: name, size, type, ownership&mode, contents
+ *	0: name, type       (symbolic links ok)
+ *	1: name, type, size (symbolic links ok)
+ *	2: name, type, size (exactly)
+ *	3: name, type, size, ownership&mode
+ *	4: name, type, size, contents
+ *	5: name, type, size, ownership&mode, contents
  */
 void
 compare_panels(int level)
@@ -232,7 +235,7 @@ compare_panels(int level)
 	/* reread panels */
 	list_both_directories();
 
-	if (level >= 3) {
+	if (level >= 4) {
 		win_waitmsg();
 		pathname_set_directory(USTR(ppanel_file->other->dir));
 	}
@@ -251,7 +254,7 @@ compare_panels(int level)
 		pfe2 = ppanel_file->other->files[i];
 		ppanel_file->other->selected += selectfile(pfe2,FN_SELECT);
 
-		/* level0: comparing name */
+		/* all levels: comparing name */
 		name2 = SDSTR(pfe2->file);
 		for (pfe1 = 0, min = 0, max = cnt1 - 1; min <= max; ) {
 			med = (min + max) / 2;
@@ -269,40 +272,34 @@ compare_panels(int level)
 		if (pfe1 == 0)
 			continue;
 
-		/* level0: comparing type */
-		if ( !( (IS_FT_PLAIN(pfe1->file_type) &&
-			IS_FT_PLAIN(pfe2->file_type)) ||
+		/* all levels: comparing type */
+		if ( !( (IS_FT_PLAIN(pfe1->file_type)
+			&& IS_FT_PLAIN(pfe2->file_type)) ||
 		  (IS_FT_DIR(pfe1->file_type) &&
 			IS_FT_DIR(pfe2->file_type)) ||
-		  (pfe1->file_type == pfe2->file_type &&
-			pfe1->file_type != FT_NA)
+		  (pfe1->file_type == pfe2->file_type
+			&& pfe1->file_type != FT_NA)
 		  ))
 			continue;
 
-		/* level0: comparing size (or device numbers) */
-		if (IS_FT_DEV(pfe1->file_type)) {
-			/* devices should have the same major/minor numbers */
-			if (pfe1->devnum != pfe2->devnum)
-				continue;
-		}
-		else if (IS_FT_PLAIN(pfe1->file_type)) {
-			/* plain files should have the same size */
-			if (pfe1->size != pfe2->size)
-				continue;
-		}
-
-		/* level 1+: symbolic links not ok */
-		if (level >= 1 && pfe1->symlink != pfe2->symlink)
+		/* level 1+: comparing size (or device numbers) */
+		if (level >= 1
+		  && ((IS_FT_DEV(pfe1->file_type) && pfe1->devnum != pfe2->devnum)
+		  || (IS_FT_PLAIN(pfe1->file_type) && pfe1->size != pfe2->size)))
 			continue;
 
-		/* level 2,4+: ownership and mode */
-		if ((level == 2 || level >= 4) && (pfe1->uid != pfe2->uid ||
-		  pfe1->gid != pfe2->gid || pfe1->mode12 != pfe2->mode12))
+		/* level 2+: symbolic links not ok */
+		if (level >= 2 && pfe1->symlink != pfe2->symlink)
 			continue;
 
-		/* level 3+: comparing data (contents) */
-		if (level >= 3 && IS_FT_PLAIN(pfe1->file_type) &&
-		  (cmp = file_cmp(SDSTR(pfe1->file),pathname_join(name2))) ) {
+		/* level 3,5+: ownership and mode */
+		if ((level == 3 || level >= 5) && (pfe1->uid != pfe2->uid
+		  || pfe1->gid != pfe2->gid || pfe1->mode12 != pfe2->mode12))
+			continue;
+
+		/* level 4+: comparing data (contents) */
+		if (level >= 4 && IS_FT_PLAIN(pfe1->file_type)
+		  && (cmp = file_cmp(SDSTR(pfe1->file),pathname_join(name2))) ) {
 			if (cmp < 0 && ++errcnt <= 3)
 				win_warning_fmt("COMPARE: Cannot read file '%s'.",
 				  name2);
@@ -328,16 +325,8 @@ compare_panels(int level)
 static void
 compare_level(int level)
 {
-	if (level >= 0) {
-		panel_compare.pd->curs = level;	/* remember level */
-		compare_panels(level);
-	}
+	compare_panels(level);
 	next_mode = MODE_SPECIAL_RETURN;
 }
 
 void cx_compare(void)   { compare_level(panel_compare.pd->curs); }
-void cx_compare_0(void) { compare_level(0); }
-void cx_compare_1(void) { compare_level(1); }
-void cx_compare_2(void) { compare_level(2); }
-void cx_compare_3(void) { compare_level(3); }
-void cx_compare_4(void) { compare_level(4); }
