@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -33,9 +33,6 @@
 #include "directory.h"	/* filepos_save() */
 #include "sdstring.h"	/* SDSTR() */
 
-/* result of CMP() is an int even if V1 and V2 are not */
-#define CMP(V1,V2) ((V1) == (V2) ? 0 : (V1) < (V2) ? -1 : 1)
-
 void
 sort_prepare(void)
 {
@@ -66,8 +63,8 @@ revstrcmp(const char *s1, const char *s2)
 	int c1, c2;
 
 	for (i1 = strlen(s1), i2 = strlen(s2); i1 > 0 && i2 > 0;) {
-		c1 = s1[--i1] & 0xFF;
-		c2 = s2[--i2] & 0xFF;
+		c1 = (unsigned char)s1[--i1];
+		c2 = (unsigned char)s2[--i2];
 		/*
 		 * ignoring LOCALE, this sort order has nothing to do
 		 * with a human language, it is intended for sendmail
@@ -176,6 +173,5 @@ sort_files(void)
 {
 	if (ppanel_file->pd->cnt == 0)
 		return;
-
 	qsort(ppanel_file->files,ppanel_file->pd->cnt,sizeof(FILE_ENTRY *),qcmp);
 }
