@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -44,47 +44,104 @@
 #include "xterm_title.h"/* xterm_title_initialize() */
 
 /* GLOBAL VARIABLES */
-PANEL_DESC *panel = 0;
+
+static EXTRA_LINE el_leave[1] = {
+	{ 0,0,MODE_SPECIAL_RETURN,0 }
+};
+static EXTRA_LINE el_bm_lst[3] = {
+	{ 0,0,MODE_SPECIAL_RETURN,0 },
+	{ "Manage bookmarks",0,MODE_BM_MANAGER,0 },
+	{ "Bookmark the current working directory",
+	  "@1" /* handled by win_info_extra_line() */,
+	  0,cx_bm_list_bookmark }
+};
+static EXTRA_LINE el_bm_mng[2] = {
+	{ 0, "Bookmarks are saved automatically",MODE_BM_LIST,cx_bm_mng_save },
+	{ "Revert the bookmarks to previous state",
+	  "All changes will be lost",0,cx_bm_mng_revert }
+};
+static EXTRA_LINE el_cfg[3] = {
+	{ "Cancel","No configuration change",MODE_SPECIAL_RETURN,0 },
+	{ "Accept","Use the new configuration in this session",
+	  MODE_SPECIAL_RETURN,cx_config_save },
+	{ "Accept+Save","Save the configuration to ~/.clexrc",
+	  MODE_SPECIAL_RETURN,cx_config_save }
+};
+static EXTRA_LINE el_cfg_admin[2] = {
+	{ "Exit",
+	  "Exit program (don't forget to save the configuration first)",
+	  MODE_SPECIAL_QUIT,0 },
+	{ "Save","Save the new system-wide defaults to " CONFIG_FILE,
+	  0,cx_config_admin_save }
+};
+/* pd_dir_split uses only the first entry of el_dir */
+static EXTRA_LINE el_dir[2] = {
+	{ 0,0,MODE_SPECIAL_RETURN,cx_dir_enter },
+	{ "Bookmarks",0,MODE_BM_LIST,cx_dir_enter }
+};
+static EXTRA_LINE el_grp[2] = {
+	{ 0,0,MODE_SPECIAL_RETURN,0 },
+	{ "Switch to user data",0,MODE_USER,0 }
+};
+static EXTRA_LINE el_help[1] = {
+	{ "HELP: Table of Contents",0,0,cx_help_contents }
+};
+static EXTRA_LINE el_usr[2] = {
+	{ 0,0,MODE_SPECIAL_RETURN,0 },
+	{ "Switch to group data",0,MODE_GROUP,0 }
+};
+
+static INPUTLINE il_filt;
+
 /*
- * initialization:
- *   cnt, top, curs, min, type, norev
- *   <panel specific data>
+ * PANEL_DESC initialization:
+ *   cnt, top, curs, min, type, norev, extra, filter, filtering
  */
-static PANEL_DESC pd_bm =
-  { 0,0,-2,-2,PANEL_TYPE_BM,0 };
-static PANEL_DESC pd_compl =
-  { 0,0,0,-1,PANEL_TYPE_COMPL,0 };
+static PANEL_DESC pd_bm_lst =
+  { 0,0,0,-3,PANEL_TYPE_BM,0,el_bm_lst,0,0};
+static PANEL_DESC pd_bm_mng =
+  { 0,0,0,-2,PANEL_TYPE_BM,0,el_bm_mng,0,0};
 static PANEL_DESC pd_cfg =
-  { CFG_VARIABLES,0,0,0,PANEL_TYPE_CFG,0 };
+  { CFG_VARIABLES,0,0,-3,PANEL_TYPE_CFG,0,el_cfg,0,0 };
+static PANEL_DESC pd_compare =
+  /* 6 items in this menu */
+  {  6,-1,2,-1,PANEL_TYPE_COMPARE,0,el_leave,0,0 };
+static PANEL_DESC pd_compl =
+  { 0,0,0,-1,PANEL_TYPE_COMPL,0,el_leave,0,0 };
 static PANEL_DESC pd_dir =
-  { 0,0,0,-2,PANEL_TYPE_DIR,0 };
+  { 0,0,0,-2,PANEL_TYPE_DIR,0,el_dir,&il_filt,0 };
 static PANEL_DESC pd_dir_split =
-  { 0,0,0,-1,PANEL_TYPE_DIR_SPLIT,0 };
+  { 0,0,0,-1,PANEL_TYPE_DIR_SPLIT,0,el_dir,0,0 };
+static PANEL_DESC pd_grp =
+  { 0,0,0,-2,PANEL_TYPE_GROUP,0,el_grp,&il_filt,0 };
 static PANEL_DESC pd_help =
-  { 0,0,0, 0,PANEL_TYPE_HELP,1 };
+  { 0,0,0,-1,PANEL_TYPE_HELP,1,el_help,0,0 };
 static PANEL_DESC pd_hist =
-  { 0,0,0,-1,PANEL_TYPE_HIST,0 };
-static PANEL_DESC pd_locate =
-  { 0,0,0, 0,PANEL_TYPE_LOCATE,0 };
+  { 0,0,0,-1,PANEL_TYPE_HIST,0,el_leave,&il_filt,0 };
 static PANEL_DESC pd_mainmenu =
   /* 20 items in this menu */
-  { 20,-1,-1,-1,PANEL_TYPE_MAINMENU,0 };
-static PANEL_DESC pd_compare =
-  /* 5 items in this menu */
-  {  5,-1,1,-1,PANEL_TYPE_COMPARE,0 };
+  { 20,-1,-1,-1,PANEL_TYPE_MAINMENU,0,el_leave,0,0 };
+static PANEL_DESC pd_paste =
+  /* 13 items in this menu */
+  { 13,-1,-1,-1,PANEL_TYPE_PASTE,0,el_leave,0,0 };
 static PANEL_DESC pd_sort =
   /* 7 items in this menu */ 
-  {  7,0,0,-1,PANEL_TYPE_SORT,0 };
-PANEL_BM panel_bm = { &pd_bm };
+  {  7,0,0,-1,PANEL_TYPE_SORT,0,el_leave,0,0 };
+static PANEL_DESC pd_usr =
+  { 0,0,0,-2,PANEL_TYPE_USER,0,el_usr,&il_filt,0 };
+PANEL_DESC *panel = 0;
+PANEL_BM panel_bm_lst = { &pd_bm_lst }, panel_bm_mng = { &pd_bm_mng };
 PANEL_COMPL panel_compl = { &pd_compl };
 PANEL_CFG panel_cfg = { &pd_cfg };
 PANEL_DIR panel_dir = { &pd_dir }, panel_dir_split = { &pd_dir_split };
+PANEL_GROUP panel_group = { &pd_grp,0,0 };
 PANEL_HELP panel_help = { &pd_help };
 PANEL_HIST panel_hist = { &pd_hist };
-PANEL_LOCATE panel_locate = { &pd_locate, 0,0 };
 PANEL_MENU panel_mainmenu = { &pd_mainmenu };
 PANEL_MENU panel_compare = { &pd_compare };
+PANEL_MENU panel_paste = { &pd_paste };
 PANEL_SORT panel_sort =  { &pd_sort, SORT_NAME };
+PANEL_USER panel_user = { &pd_usr,0,0 };
 PANEL_FILE *ppanel_file;
 DISPLAY display = { 0,0,0,0,0,0,0 };
 CLEX_DATA clex_data;
@@ -131,7 +188,7 @@ main(int argc, char *argv[])
 #endif
 		  "  system-wide configuration file:   " CONFIG_FILE "\n\n"
 
-		  "Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>"
+		  "Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>"
 		  "\n\n"
 		  "This is free software distributed without any warranty.\n"
 		  "See the GNU General Public License for more details.\n"
@@ -184,6 +241,11 @@ main(int argc, char *argv[])
 		list_initialize();
 		xterm_title_initialize();
 	}
+	else {
+		/* modify the configuration panel for use in the admin mode */
+		pd_cfg.min = -2;
+		pd_cfg.extra = el_cfg_admin;
+	}
 
 	/* start CURSES screen handling */
 	tty_initialize();
