@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -16,6 +16,7 @@
 
 #include <sys/types.h>	/* pid_t */
 #include <errno.h>		/* errno */
+#include <fcntl.h>		/* fcntl */
 #include <stdio.h>		/* fputs() */
 #include <termios.h>	/* struct termios */
 #include <unistd.h>		/* STDIN_FILENO */
@@ -80,15 +81,26 @@ tty_pgrp_reset(void)
 void
 tty_press_enter(void)
 {
-	int in;
+	int in, flags;
 
 	fputs("Press <enter> to continue. ",stdout);
 	fflush(stdout);
 	tty_setraw();
 	while ((in = getchar()) != '\n' && in != '\r') {
+		if (in != EOF)
+			continue;
+		if (errno == EINTR)
+			continue;
+		if (errno == EAGAIN) {
+			flags = fcntl(STDIN_FILENO,F_GETFL);
+			if ((flags & O_NONBLOCK) == O_NONBLOCK) {
+				/* clear the non-blocking flag */
+				fcntl(STDIN_FILENO,F_SETFL,flags & ~O_NONBLOCK);
+				continue;
+			}
+		}
 		/* prevent looping in the case of an error */
-		if (in == EOF && errno != EINTR)
-			err_exit("Cannot read from standard input");
+		err_exit("Cannot read from standard input");
 	}
 	tty_reset();
 
