@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -104,7 +104,7 @@ qcmp_name(const void *e1, const void *e2)
 static int
 qcmp_uid(const void *e1, const void *e2)
 {
-	return (*(PWDATA **)e1)->uid - (*(PWDATA **)e2)->uid;
+	return CMP((*(PWDATA **)e1)->uid,(*(PWDATA **)e2)->uid);
 }
 
 static void
@@ -170,7 +170,7 @@ read_utable(void)
 static int
 qcmp_gid(const void *e1, const void *e2)
 {
-	return (*(GRDATA **)e1)->gid - (*(GRDATA **)e2)->gid;
+	return CMP((*(GRDATA **)e1)->gid,(*(GRDATA **)e2)->gid);
 }
 
 static void
@@ -277,14 +277,12 @@ userdata_expire(void)
 	 utable.timestamp = gtable.timestamp = 0;
 }
 
-/*
- * refresh the cached user/group data
- * returns 1 if data was re-read, 0 if unchanged
- */
+/* returns 1 if data was re-read, 0 if unchanged */
 int
 userdata_refresh(void)
 {
-	FLAG stat_ok, reloaded;
+	FLAG stat_ok;
+	int reloaded;
 	struct stat st;
 
 	reloaded = 0;
@@ -336,7 +334,7 @@ const char *
 lookup_login(uid_t uid)
 {
 	BIN_SEARCH(utable.cnt,
-	  uid - utable.by_uid[med]->uid,
+	  CMP(uid,utable.by_uid[med]->uid),
 	  SDSTR(utable.by_uid[med]->login))
 }
 
@@ -345,7 +343,7 @@ const char *
 lookup_group(gid_t gid)
 {
 	BIN_SEARCH(gtable.cnt,
-	  gid - gtable.by_gid[med]->gid,
+	  CMP(gid,gtable.by_gid[med]->gid),
 	  SDSTR(gtable.by_gid[med]->group))
 }
 
@@ -449,3 +447,89 @@ username_find(const char **pfullname)
 	find.index++;
 	return login;
 }
+
+void
+user_panel(void)
+{
+	int i, j;
+	const char *filter, *login, *gecos;
+	uid_t curs;
+
+	if (utable.cnt > panel_user.usr_alloc) {
+		panel_user.usr_alloc = utable.cnt;
+		if (panel_user.users)
+			free(panel_user.users);
+		panel_user.users = emalloc(panel_user.usr_alloc * sizeof(USER_ENTRY));
+	}
+
+	curs = VALID_CURSOR(panel_user.pd) ?
+	  panel_user.users[panel_user.pd->curs].uid : 0;
+	filter = panel_user.pd->filtering ? panel_user.pd->filter->line : 0;
+
+	for (i = j = 0; i < utable.cnt; i++) {
+		if (curs == utable.by_uid[i]->uid)
+			panel_user.pd->curs = j;
+		login = SDSTR(utable.by_uid[i]->login);
+		gecos = SDSTR(utable.by_uid[i]->fullname);
+		if (filter && !substring(login,filter,0) && !substring(gecos,filter,1))
+			continue;
+		panel_user.users[j].uid = utable.by_uid[i]->uid;
+		panel_user.users[j].login = login;
+		panel_user.users[j++].gecos = gecos;
+	}
+
+	panel_user.pd->cnt = j;
+}
+
+void
+user_prepare(void)
+{
+	panel_user.pd->filtering = 0;
+	user_panel();
+	panel_user.pd->top = panel_user.pd->min;
+	panel_user.pd->curs = 0;
+	panel = panel_user.pd;
+	textline = 0;
+}
+
+void
+group_panel(void)
+{
+	int i, j;
+	const char *filter, *group;
+	gid_t curs;
+
+	if (gtable.cnt > panel_group.grp_alloc) {
+		panel_group.grp_alloc = gtable.cnt;
+		if (panel_group.groups)
+			free(panel_group.groups);
+		panel_group.groups = emalloc(panel_group.grp_alloc * sizeof(GROUP_ENTRY));
+	}
+
+	curs = VALID_CURSOR(panel_group.pd) ?
+	  panel_group.groups[panel_group.pd->curs].gid : 0;
+	filter = panel_group.pd->filtering ? panel_group.pd->filter->line : 0;
+
+	for (i = j = 0; i < gtable.cnt; i++) {
+		if (curs == gtable.by_gid[i]->gid)
+			panel_group.pd->curs = j;
+		group = SDSTR(gtable.by_gid[i]->group);
+		if (filter && !substring(group,filter,0))
+			continue;
+		panel_group.groups[j].gid = gtable.by_gid[i]->gid;
+		panel_group.groups[j++].group = group;
+	}
+
+	panel_group.pd->cnt = j;
+}
+
+void
+group_prepare(void)
+{
+	panel_group.pd->filtering = 0;
+	group_panel();
+	panel_group.pd->top = panel_group.pd->min;
+	panel_group.pd->curs = 0;
+	panel = panel_group.pd;
+	textline = 0;
+}
