@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2005 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2006 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -16,6 +16,7 @@
 
 #include <sys/types.h>			/* clex.h */
 #include <sys/stat.h>			/* fstat() */
+#include <ctype.h>				/* tolower */
 #include <fcntl.h>				/* open() */
 #include <stdio.h>				/* sprintf() */
 #include <stdlib.h>				/* malloc() */
@@ -34,8 +35,8 @@ static USTRING sf_buff = { 0,0 };
 static size_t sf_dirlen;
 
 /* get rid of directory part in 'pathname' */
-const char
-*base_name(const char *pathname)
+const char *
+base_name(const char *pathname)
 {
 	const char *base, *pch;
 	char ch;
@@ -46,6 +47,26 @@ const char
 	return base;
 }
 
+int
+substring(const char *haystack, const char *needle, int ic)
+{
+	const char *h, *n;
+
+	for ( ; ; haystack++ ) {
+		for (h = haystack, n = needle; ; h++, n++) {
+			if (*n == '\0')
+				return 1;
+			if (*h == '\0')
+				return 0;
+			if (ic ? tolower((unsigned char)*h) != tolower((unsigned char)*n)
+			  : *h != *n)
+				break;
+		}
+	}
+	/* NOTREACHED */
+	return 0;
+}
+
 static void
 alloc_fail(size_t size)
 {
